/*

Copyright c 2013 Xyleme, Inc., 1881 9th Street, Suite 300, Boulder, Colorado 80302 USA.
All rights reserved.

This file and related documentation are protected by copyright and
are distributed under licenses regarding their use, copying, distribution,
and decompilation. No part of this product or related documentation may
be reproduced or transmitted in any form or by any means, electronic or
mechanical, for any purpose, without the express written permission of
Xyleme, Inc.

*/


// --------------------------------------------------
// JS functions for BranchingMultipleChoice (Branching Simulation) question
// --------------------------------------------------
//
// BMCQuestion() : definition of Object type;
//   getParamValue() : get value of specified question parameter;
//   setParam() : set value for specified question parameter;
//   redraw() : redraw the question according the current data stored in question object; 
//   setUserResult() : store user answers;
// addBMCQuestion() : add question to question array;
// getBQuestion() : get question from question array;
// setBMCQuestions() : set parameters for all questions in array;
//
// handleBMCActivity() : evaluate user answer and invoke proper action;

// initBMCPage() : init the CRE object;

// --------- following functions moved to questions.js ----------
// show() : show element of question form (image, feedback...);
// hide() : hide element of question form;
// setDisabled() : disable whole question;
// setAttemptLimit() : set number of attempts remaining;
// getHint() : get question hint if available;
// getUserResults() : update question object with any existing data present in CRE;
// setUserResults() : set user results to CRE;
//

// -----------------------------------------------------------------------------------------------------------

// this array should be filled from the XSL script
var arrElements = new Array();

var j$ = jQuery.noConflict();

function hideAllExceptId(id) {

	var stemPlaceholder = document.getElementById("StemPlaceholder");
	for(i in arrElements) {
		elemId = arrElements[i];
		var elem = document.getElementById(elemId);		
		if (elem != null && typeof(elem) != 'undefined') {
			
			// hide element if it is not specified one
			if (elemId != id) {
				elem.style.display = "none";
			} else {
				// show the element
				elem.style.display = "block";
				// show question stem
				if (stemPlaceholder != null && typeof(stemPlaceholder) != 'undefined') {
					var stem = document.getElementById("InnerQuestionStem_" + id);
					if (stem != null && typeof(stem) != 'undefined') {
						stemPlaceholder.innerHTML = stem.innerHTML;
					}
				}
			}
		}
	}
}

function simulateClick(elId) {
	var evt;
	var el = document.getElementById(elId);
	if (document.createEvent) {
		evt = document.createEvent("MouseEvents");
		evt.initMouseEvent("click", true, true, window,
			0, 0, 0, 0, 0, false, false, false, false, 0, null);
	}
	(evt)? el.dispatchEvent(evt):(el.click && el.click());
}

function replayBranchingSim() {
	// show main Branching Sim content
	swapQuestionOutcomeMode(false);
	
	if (typeof(rootID) != "undefined" && rootID != null) 
		hideAllExceptId(rootID);
}

function selectOption(id) {
	var option = document.getElementById(id);
	if (option != null && typeof(option) != 'undefined') { 
		option.checked = true;
	}
}

function showOutcomePopup(element) {
	
	if (typeof(element.id) != 'undefined' && element.id.indexOf('_outcome_') > -1) {
		// get source text of outcone
		var outcomePopup = document.getElementById(element.id + "_content");		
		if (outcomePopup != null && typeof(outcomePopup) != 'undefined') {
			
			// show outcome content
			swapQuestionOutcomeMode(true);
			
			// copy HTML to popup div placeholder
			var outcomePlaceholder = document.getElementById("OutcomeContentPlaceholder");
			if (outcomePlaceholder != null && typeof(outcomePlaceholder) != 'undefined') { 	
				outcomePlaceholder.innerHTML = outcomePopup.innerHTML;
				simulateClick('ShowOutcome');
			}
		}
	}
}

function swapQuestionOutcomeMode(isOutcomeMode) {
	var branchingSimMainTable = document.getElementById("BranchingSimMainTable");
	var branchingSimOutcomeMainTable = document.getElementById("BranchingSimOutcomeMainTable");	
		
	if (branchingSimMainTable != null && typeof(branchingSimMainTable) != 'undefined' && 
		branchingSimOutcomeMainTable != null && typeof(branchingSimOutcomeMainTable) != 'undefined') { 	
			if (isOutcomeMode == true) {	
				branchingSimMainTable.style.display = "none";
				branchingSimOutcomeMainTable.style.display = "block";
			} else {
				branchingSimMainTable.style.display = "block";
				branchingSimOutcomeMainTable.style.display = "none";
			}
	}

}

function submitOption(optionId, qid, multipleChoiceId, errorMsgText, title) {

    if(!inited) {
       setTimeout("submitOption('" + optionId + "', '" + qid + "', '" + multipleChoiceId + "', '" + errorMsgText + "', '" + title + "')", 1000);
       return;
    }
    
	selectOption(optionId);
	handleBMCActivity(qid, multipleChoiceId, optionId, errorMsgText, title);

}

function setOutsetOptionStyle(id) {
	var option = document.getElementById(id);
	if (option != null && typeof(option) != 'undefined') { 
		option.style.borderStyle = "outset";
		//option.style.color = "black";
	}
	
}

function setInsetOptionStyle(id) {
	
	var option = document.getElementById(id);
	if (option != null && typeof(option) != 'undefined') { 
		option.style.borderStyle = "inset";
		//option.style.color = "#3578FF";
	}
	
}

// -----------------------------------------------------------------------------------------------------------

scorableContent = true;   //Used by CRE to determine if the page is scorable
BMCQ = true;               // defined in questions.js
var bmcquestions = new Array();
var inited = false;

// ---------------------------------
// --- definition of Object type ---
function BMCQuestion()
{
    this.randomize = false;
    this.redraw = function ()
        {   
        }
}

// Extend basic Question type
BMCQuestion.prototype = new Question;

// --------------------------------------
// --- add question to question array ---
function addBMCQuestion(question)
{
    //    alert("addBMCQ: "+question);
    bmcquestions[bmcquestions.length] = question;
    addQuestion(question);
}

// -------------------------------------------------
// --- set parameters for all questions in array ---
function setBMCQuestions()
{
    for (l=0; l<document.forms.length; l++) {
        var fname = document.forms[l].name;     
        if (fname.toLowerCase().indexOf("questionform") == -1) continue;
        var fid = document.forms[l].id;
        if (fid.toLowerCase().indexOf("rootbranchingmultiplechoice") == -1) continue;

        var wab_id = fname.substring(13, fname.indexOf("_mc_"));
        
        //var wab_obj = document.getElementById("wab_"+wab_id);
        newBMCQuestion = new BMCQuestion();
        newBMCQuestion.setParam("guid", wab_id);
        //var params = wab_obj.attributes;    
        //var param;

        newBMCQuestion.setParam("randomize", "true");
        newBMCQuestion.setParam("attempts", "1");                
        newBMCQuestion.setParam("showHint", "false");
        //newBMCQuestion.setParam("showFeedback", 
        //        wab_obj.getAttribute("wab_ShowFeedback"));      
        //newBMCQuestion.setParam("hintActive", 
        //        wab_obj.getAttribute("wab_hint_active"));
        newBMCQuestion.setParam("corectAnswer", "1");
        //newBMCQuestion.setParam("rollupWeight", 
        //        wab_obj.getAttribute("wab_RollupWeight"));

        addBMCQuestion(newBMCQuestion);
    }
}


function handleBMCActivity(qid, multipleChoiceId, optionId, msg, title) {
	var currentQuestion = getQuestion(qid);
	
	var isSelected = false;    //  determines if user selected some radio element
	var user_answer_radio = "";	
	 
	var allElemsInForm = document.forms["QuestionForm_" + multipleChoiceId].elements;
	
	var radio = null;
	for (var i=0; i < allElemsInForm.length; i++) {
		if (typeof(allElemsInForm[i]) == "object" && allElemsInForm[i].checked != "") {
			isSelected = true;
			user_answer_radio = allElemsInForm[i].value;
		}
	}	
	
	if (isSelected) {
		var subBranchId = getNextQuestion(qid, multipleChoiceId, user_answer_radio);
		var content = j$('#' + subBranchId + "_content div div").html();
		if (content != null && content.toLowerCase() == "retry"){				// workaround for SALES-32 (if Outcome has "retry" word only than show feedback without Outcome open)
			showFeedBack(optionId);
			return;
		}
	
		subBranchId = showNextQuestion(qid, multipleChoiceId, user_answer_radio);		
		var answer = checkUserAnswer(subBranchId);

		//this is assuming we are in the outcome of the Branching Simulation tree (not in a middle MultipleChoice)			
		if (answer != null) {
			submitScore(qid, answer);
			
			currentQuestion.setParam("userAnswer", user_answer_radio);
			
			currentQuestion.setUserResult();   //send result to CRE here
			setUserResults();
		}		
		
		// feedback
		if (getRequestParameter("mode") == "review"){
			var sfeedback = "true";
		} else {
			var sfeedback = currentQuestion.getParamValue("showFeedback");
		}
		  
		// TODO  operate with feedback
		// show feed back of previous activity			
		showFeedBack(optionId); //showFeedBack(multipleChoiceId);		
		
		// show outcome popup
		var elem = document.getElementById(subBranchId);		
		if (elem != null && typeof(elem) != 'undefined') {
			showOutcomePopup(elem);
		}
		
	} else {
		// show error message 
		if (msg)
			jAlert(msg, title);
	}
	
	
}

function showNextQuestion(qid, multipleChoiceId, user_answer_radio) {		
	var found = false;
	
	var position;		
	for (position = 0; position < arrAssociations.length; position++) {
		if (arrAssociations[position] == user_answer_radio) {
			found = true;
			break;
		}		
	}
	
	// go to next element in the associations array
	position++;
	
	if (position < arrAssociations.length) {
		var subBranchId = arrAssociations[position];
		// go to next question or outcome
		hideAllExceptId(subBranchId);
		return subBranchId;
	}
	return "";
}

function getNextQuestion(qid, multipleChoiceId, user_answer_radio) {		
	for (var position = 0; position < arrAssociations.length; position++) {
		if (arrAssociations[position] == user_answer_radio) {
			return arrAssociations[++position];
		}		
	}
	return "";
}

function showFeedBack(optionId) {
	// get feedbackId
	var feedbackId = optionId + "_option_feedback";
	
	var feedbackPlaceholder = document.getElementById("FeedbackPlaceholder");
	if (feedbackPlaceholder != null && typeof(feedbackPlaceholder) != 'undefined') { 	
		var feedback = document.getElementById(feedbackId);
		if (feedback != null && typeof(feedback) != 'undefined') {
			j$(feedbackPlaceholder).css({'display':'none'});
			feedbackPlaceholder.innerHTML = feedback.innerHTML;
			j$(feedbackPlaceholder).animate({"opacity": "show"}, { duration: 2000});
		} else {
			feedbackPlaceholder.innerHTML = "";
			j$(feedbackPlaceholder).hide('fast');
		}
	}
}

/** 
	Where user gets into outcome - check his answer.
*/
function checkUserAnswer(subBranchId) {		
	// suppose we have Outcome object
	var answObj = subBranchId + "_a";
	var elem = document.getElementById(answObj);
	if (elem != null && typeof(elem) != 'undefined') {
		if (elem.value == "1")
			return true;
		else
			return false;
	}
	return null;
}

function submitScore(qid, answer) {
	var currentQuestion = getQuestion(qid);
	
	if (answer) {
		currentQuestion.setParam("attempts", 0); // there is only one attempt
		currentQuestion.setParam("result", "correct"); // set question result
	} else if (answer == false) {
		currentQuestion.setParam("attempts", 0); // there is only one attempt
		currentQuestion.setParam("result", "incorrect"); // set question result
	} else {
		// not an outcome
	}
	
	//alert(answer);
}

// -------------------------------
// --- init the whole CRE page ---
function initBMCPage()
{   
    //    alert("ga-initMCPage()");
    setBMCQuestions();
    getUserResults(bmcquestions);
    inited = true;
    var contentCover = document.getElementById('ContentCover');
    if(contentCover) {
        contentCover.style.visibility = 'hidden';
		contentCover.style.display = 'none';
    }
}

//window.onload = initQuestionPage();