/*

Copyright c 2013 Xyleme, Inc., 1881 9th Street, Suite 300, Boulder, Colorado 80302 USA.
All rights reserved.

This file and related documentation are protected by copyright and
are distributed under licenses regarding their use, copying, distribution,
and decompilation. No part of this product or related documentation may
be reproduced or transmitted in any form or by any means, electronic or
mechanical, for any purpose, without the express written permission of
Xyleme, Inc.

*/

 
// --------------------------------------------------
// JS functions for WebFillInBlank question
// --------------------------------------------------
//
// FBQuestion() : definition of Object type;
//   getParamValue() : get value of specified question parameter;
//   setParam() : set value for specified question parameter;
//   redraw() : redraw the question according the current data stored in question object; 
//   setUserResult() : store user answers;
// addFBQuestion() : add question to question array;
// getQuestion() : get question from question array;
// setFBQuestions() : set parameters for all questions in array;
//
// handleWebFibActivity() : evaluate user answer and invoke proper action;
// checkFIBSubmit() : check if submit is valid;
// show() : show element of question form (image, feedback...);
// hide() : hide element of question form;
// setDisabled() : disable whole question;
// setAttemptLimit() : set number of attempts remaining;
// getHint() : get question hint if available;
// getUserResults() : update question object with any existing data present in CRE;
// setUserResults() : set user results to CRE;
// initPage() : init the CRE object;
//

// lines marked with xxx are commented/uncommented depending on onCRE/offCRE environment


scorableContent = true;   //Used by CRE to determine if the page is scorable
DWA = true;               // defined in questions.js
var dwactivities = new Array();
var dwa_guid;

var stepHeights = new Array();

var comment_window = null;
var windowIsOpen = false;
var popupPath = "";
var entrys = new Array();
var generateButton = null;

// ---------------------------------
// --- definition of Object type ---
function DWActivity()
{
    this.redraw = function ()
	 {
	 
		var user_anws;
		
		if (getRequestParameter("mode") == "review"	&& navigator.appName != "Microsoft Internet Explorer" ) {
			user_anws = (utf8Decode(unescape(this.userAnswer))).split("||");
		} else {
			user_anws = unescape(this.userAnswer).split("||");
		}
	 
		var fields = document.getElementsByTagName("textarea");
		var j = 0;
		for(var i = 0; fields.length > i; i++){
			if (fields[i].id.indexOf('entryField') != -1){
				fields[i].value = user_anws[j++];
				checkLen(fields[i], fields[i].name);
			}
		}
		
	}
}

// Extend basic Question type
DWActivity.prototype = new Question;

// --------------------------------------
// --- add question to question array ---
function addDWActivity(question)
{
    dwactivities[dwactivities.length] = question;
    addQuestion(question);
}

// -------------------------------------------------
// --- set parameters for all dwactivities in array ---
function setDWActivities()
{
	var dwa_obj = document.getElementById("DocWriterActivity");
	dwa_guid = dwa_obj.getAttribute('name');
	newActivity = new DWActivity();
	newActivity.setParam("guid", dwa_guid);
	newActivity.setParam("attempts", "" + 100); 
    addDWActivity(newActivity);
}

// -------------------------------
// --- init the whole CRE page ---

function initDocWriterPage()
{   
    setDWActivities();
    getUserResults(dwactivities);
	document.body.onunload = saveText;
}

function saveText(qid){
	var user_anw = "";
    
	var fields = document.getElementsByTagName("textarea");
	for(var i = 0; fields.length > i; i++){
		if (fields[i].id.indexOf('entryField') != -1){
			user_anw += fields[i].value + "||";
		}
	}
	
	getQuestion(dwa_guid).setParam("userAnswer", user_anw);
	getQuestion(dwa_guid).setParam("result", "incorrect");
	getQuestion(dwa_guid).setUserResult();
	APIsetValue("cmi.completion_status", "completed");
	//setUserResults();
	
	API.Terminate("");
	
	return true;
}

function showHideStep(id) {
	var step = document.getElementById(id);
    if(step.down) {
		step.glidetime=1000;
        step.down = false;
		slideengine(step, "up", new Date().getTime());
    } else {
        step.glidetime=1000;
        step.down = true;
         if(document.getElementById(id + "Body").offsetHeight > stepHeights[id])
            stepHeights[id] = document.getElementById(id + "Body").offsetHeight;
		slideengine(step, "down", new Date().getTime());
        for(i in stepHeights) {
            if(i != id && document.getElementById(i).down) {
                stepI = document.getElementById(i);
                stepI.glidetime=1000;
                stepI.down = false;
                slideengine(stepI, "up", new Date().getTime());
            }
        }
	}
}
	
function slideengine(obj, direction, startTime){
	var elapsed=new Date().getTime()-startTime //get time animation has run
	if (elapsed<obj.glidetime){ //if time run is less than specified length
		var distancepercent=(direction=="down")? curveincrement(elapsed/obj.glidetime, false) : curveincrement(elapsed/obj.glidetime, true)
		var currentclip=(distancepercent*stepHeights[obj.id])+"px"
		obj.style.height = currentclip;
		window["glidetimer_"+obj.id]=setTimeout(function(){slideengine(obj, direction, startTime)}, 10)
	}
	else{ //if animation finished
		if (direction=="up") {
            obj.style.height = "0px";
		}
	}
}

function curveincrement(percent, hide){
	if (hide)
		return (Math.cos(percent*Math.PI/2))  //return cos curve based value from a percentage input
	else 
		return (1 - Math.cos(percent*Math.PI/2)) //return cos curve based value from a percentage input
}

function clearfields(){
	var check;
	var langStr = document.langDocWriterClearfields;
	//check = confirm('Are you sure you want to clear the form and start over?');
	check = confirm(langStr);
	if (check){
		var fields = document.getElementsByTagName("textarea");
		for(var i = 0; fields.length > i; i++){
			if (fields[i].id.indexOf('entryField') != -1){
				fields[i].value = "";
				checkLen(fields[i], fields[i].name);
			}
		}
	}
}

function generateDocument(path) {
	popupPath = path;
	var width = 800;
    var height = 600;
    var left = parseInt((screen.availWidth/2) - (width/2));
    var top = parseInt((screen.availHeight/2) - (height/2));
	if (comment_window == null){
	comment_window = window.open(path,'printwindow','width='+width+',height='+height+',toolbar=no,left=' + left + ',top=' + top + ',location=no,directories=no,status=no,menubar=no,scrollbars=yes,copyhistory=no,resizable=no', true);
		addEvent(comment_window, 'unload', windowClose);
		addEvent(comment_window, 'load', fillBlanks);
	}else
		fillBlanks();
	comment_window.window.focus();	
}

function fillBlanks(){
	try{
		var fieldsDiv =  comment_window.document.getElementsByTagName("div");
	}
	catch(err){
		comment_window = null;
		generateDocument(popupPath);
	}
	var fieldsTextarea = document.getElementsByTagName("textarea");
	for(var i = 0; fieldsTextarea.length > i; i++){
		if (fieldsTextarea[i].id.indexOf('entryField') != -1)
			for (var j = 0; fieldsDiv.length > j; j++){
				var field = fieldsDiv[j];
				if (field.id.indexOf('blank') != -1){
					var indexStr = field.id.substring(5);
					if ((i+1) == parseInt(indexStr))
						field.innerHTML = fieldsTextarea[i].value;
				}
			}
	}
	comment_window.document.getElementById("printItem").onclick = printItem;
	windowIsOpen=true;	
	if (comment_window.event){
		comment_window.document.onkeyup = keyCheck;
		comment_window.document.onkeydown = keyCheck;
	}
	comment_window.document.onkeypress = keyCheck;
}

function keyCheck(e){
    var keyId = (comment_window.event) ? comment_window.event.keyCode : e.keyCode;
	if(keyId == 116)    {
         if (comment_window.event) comment_window.event.keyCode = 0;
         else keyId = 0;
         return false;
    }
}

function windowClose(){
	if (windowIsOpen){
		comment_window = null;
		windowIsOpen=false;
	}
}

function entryFocusLost(){
	if (comment_window != null)
		fillBlanks();
}

function printItem(){
	comment_window.print();
}
function checkLen(component, maxLen){
	if (entrys.length == 0){
		var j = 0;
		var fields = document.getElementsByTagName("textarea");
		for(var k = 0; fields.length > k; k++){
			if (fields[k].id.indexOf('entryField') != -1){
				entrys[j] = fields[k];
				j++;
			}
		}
		generateButton = document.getElementById('generateButton');
	}
	var isEnabled = true;
	for(var i = 0; entrys.length > i; i++){
		isEnabled = isEnabled && (entrys[i].value != '');
	}
	if (isEnabled)
		generateButton.removeAttribute("disabled");
	else
		generateButton.setAttribute('disabled', !isEnabled);
	
	var symbols = "characters";
	var indexStr = component.id.substring(10);
	var len = component.value.length;
	if ( len > maxLen){ 
		component.value = component.value.substring(0,maxLen);
		len = component.value.length;
	}
	count = maxLen - len;
	var entryFieldDiv = document.getElementById("entryFieldDiv" + indexStr);
	if ( 2 > count ) symbols = "character";
	entryFieldDiv.innerHTML = count + ' ' + symbols + ' left';
}

function addEvent(obj, evType, fn) {
    if (obj.addEventListener) {
        obj.addEventListener(evType, fn, true);
        return true;
    } else if (obj.attachEvent) {
        var r = obj.attachEvent("on"+evType, fn);
        return r;
    } else {
        return false;
    }
}