 /*

Copyright c 2013 Xyleme, Inc., 1881 9th Street, Suite 300, Boulder, Colorado 80302 USA.
All rights reserved.

This file and related documentation are protected by copyright and
are distributed under licenses regarding their use, copying, distribution,
and decompilation. No part of this product or related documentation may
be reproduced or transmitted in any form or by any means, electronic or
mechanical, for any purpose, without the express written permission of
Xyleme, Inc.

*/

 
// --------------------------------------------------
// JS functions for SelectHotText question
// --------------------------------------------------
//
// 	 EAQuestion() : definition of Object type;
//   getParamValue() : get value of specified question parameter;
//   setParam() : set value for specified question parameter;
//   redraw() : redraw the question according the current data stored in question object; 
//   setUserResult() : store user answers;
// 	 addEAQuestion() : add question to question array;
// 	 getQuestion() : get question from question array;
// 	 setEAQuestions() : set parameters for all questions in array;
// 	 initEAPage() : init the CRE object;



scorableContent = true;   //Used by CRE to determine if the page is scorable
EA = true;               // defined in questions.js
var eaquestions = new Array();

// ---------------------------------
// --- definition of Object type ---
function EAQuestion() {
    this.randomize = false;
    this.redraw = function () {
		if (this.attempts == 0 || this.result == "correct") {
			this.attempts = 0;
			setAttemptLimit(this.guid, 0);
			
			var input = document.getElementById("Response_" + this.guid);
			input.value = this.userAnswer;
			input.disabled = "disabled";
			setDisabled(this.guid);
		}
    }
	
	this.replaceString;
	this.setSelectionRange = q_setSelectionRange;
	this.setCaretToPos = q_setCaretToPos;
	this.getCaretPosition = q_getCaretPosition;
	this.copySelection = q_copySelection;
	this.pasteSelection = q_pasteSelection;
	this.cutSelection = q_cutSelection;	
}

// Extend basic Question type
EAQuestion.prototype = new Question;

// --------------------------------------
// --- add question to question array ---
function addEAQuestion(question) {
    eaquestions[eaquestions.length] = question;
    addQuestion(question);
}

// -------------------------------------------------
// --- set parameters for all questions in array ---
function setEAQuestions() {
    for (var l=0; l<document.forms.length; l++) {
        var fname = document.forms[l].name;     
        if (fname.toLowerCase().indexOf("questionform") == -1) continue;
        var fid = document.forms[l].id;
        if (fid.toLowerCase().indexOf("essayactivity") == -1) continue;

        var wab_id = fname.substr(13);
        
        var wab_obj = document.getElementById("wab_"+wab_id);
        newEAQuestion = new EAQuestion();
        newEAQuestion.setParam("guid", wab_id);
		newEAQuestion.setParam("attempts", 1); // set remaining attempts
        newEAQuestion.setParam("showFeedback", true);      
        newEAQuestion.setParam("rollupWeight", 1);
		
        addEAQuestion(newEAQuestion);
		
		var response = document.getElementById("Response_" + wab_id);
		
		if (response) {
			addEvent(response,'keydown', function(event){
					 if (event.ctrlKey) { 
						if (event.preventDefault) {	
							if(event.cancelable){
								if ((event.ctrlKey && event.keyCode == event.DOM_VK_V)
								|| (event.shiftKey && event.keyCode == event.DOM_VK_INSERT)) {
									var target = event.srcElement || event.target;
									if (target.nodeName == "TEXTAREA") {
										event.preventDefault();
										event.stopPropagation(); 
										return false; // The W3C DOM way			 
										
									}
								} else {
									event.preventDefault();
									event.stopPropagation(); 
									return;
								}			
							}			
						} else {
							event.returnValue = false; // The IE way
						}
					}
				});
			}
    }
}

// -------------------------------
// --- init the whole CRE page ---

function initEAPage() {   
    setEAQuestions();
    getUserResults(eaquestions);
}

/*
* analogue of handleXXXActivity() in other questions(activities)
*/
function saveEssayData(id, redraw) {
	var input = document.getElementById(id);	
	var qid = id.substr(9);
	var currentQuestion = getQuestion(qid); 
	var at_limit = currentQuestion.getParamValue("attempts");
	//currentQuestion.setParam("attempts", 0); // set remaining attempts
	currentQuestion.setParam("userAnswer", jQuery.trim(input.value));
	currentQuestion.setParam("result", "correct"); 
	
	input.disabled = "disabled";
	setDisabled(qid);
	
	setAttemptLimit(qid, at_limit);
	
	if (!redraw){
		currentQuestion.setUserResult();//send result to CRE here!
		setUserResults();
	}
}

/* 
*  Functions 
*  to apply cut, past and copy functionality 
*  via question object 
*/	
function q_cutSelection (id) {
	var input = document.getElementById(id);
	
	if(input.setSelectionRange)	{ //FF
		var selectionStart = input.selectionStart;
		var selectionEnd = input.selectionEnd;
		this.replaceString = input.value.substring(selectionStart, selectionEnd);
		input.value = input.value.substring(0, selectionStart) + input.value.substring(selectionEnd);		
    
		if (selectionStart != selectionEnd) {
			this.setSelectionRange(input, selectionStart, selectionStart);				
		} else {// set caret
			this.setCaretToPos(input, selectionStart);
			this.replaceString = "";
		}
	} else if (document.selection) {  //IE
		var range = document.selection.createRange();
		var selectionStart = this.getCaretPosition(input);
		if (range.parentElement() == input) {
			this.replaceString = range.text;
			var isCollapsed = range.text == '';
			range.text = '';
			
			if(!isCollapsed) { // there has been a selection
				range.select();
			} else {
				this.setCaretToPos(input, selectionStart);
			}
		}
	}
}

function q_copySelection(id) {
	var input = document.getElementById(id);
	
	if (input.setSelectionRange) { //FF
		var selectionStart = input.selectionStart;
		var selectionEnd = input.selectionEnd;
		this.replaceString = input.value.substring(selectionStart, selectionEnd); 	
		this.setSelectionRange(input, selectionStart, selectionEnd)
	} else if (document.selection) { //IE
		var range = document.selection.createRange();
		this.replaceString = range.text;
		
		if (range.text != "") {
				var stored_range = range.duplicate();
				stored_range.moveToElementText(input);
				stored_range.setEndPoint('EndToEnd', range );
				selectionStart  = stored_range.text.length - range.text.length;
				selectionEnd = selectionStart + range.text.length;
			} else  {
				selectionStart  = selectionEnd	= this.getCaretPosition(input);
			} 
			
		this.setSelectionRange(input, selectionStart, selectionEnd);
	}
}

function q_pasteSelection(id) {
	var input = document.getElementById(id);
	
	if (this.replaceString == null || this.replaceString == ''){
		input.focus();
		return;
	}

	if (input.setSelectionRange) { // FF
		var selectionStart = input.selectionStart;
		var selectionEnd = input.selectionEnd;
		input.value = input.value.substring(0, selectionStart) + this.replaceString + input.value.substring(selectionEnd);
		
		if (selectionStart != selectionEnd) { //  there has been a selection
			this.setSelectionRange(input, selectionStart, selectionStart + this.replaceString.length);
			this.setCaretToPos(input, selectionStart + this.replaceString.length);
		} else {// set caret
			this.setCaretToPos(input, selectionStart + this.replaceString.length);
		}
	} else if (document.selection) { //IE
			var range = document.selection.createRange();

			if (range.text != "") {
				var stored_range = range.duplicate();
				stored_range.moveToElementText(input);
				stored_range.setEndPoint('EndToEnd', range );
				selectionStart  = stored_range.text.length - range.text.length;
				selectionEnd = selectionStart + range.text.length;
			} else  {
				selectionStart  = selectionEnd	= this.getCaretPosition(input);
			} 
			
			input.value = input.value.substring(0, selectionStart) + this.replaceString + input.value.substring(selectionEnd);
			this.setCaretToPos(input, selectionStart + this.replaceString.length);
	}
}

function q_setSelectionRange(input, selectionStart, selectionEnd) {
	if (input.setSelectionRange) {
	  input.focus();
	  input.setSelectionRange(selectionStart, selectionEnd);
	} else if (input.createTextRange) {
	var range = input.createTextRange();
		range.collapse(true);
		range.moveEnd('character', selectionEnd);
		range.moveStart('character', selectionStart);
		range.select();
	}
}

function q_getCaretPosition (input) {
	var caretPos = 0;	// IE Support
	if (document.selection) {
		input.focus();
		var sel = document.selection.createRange ();
		var sel2 = sel.duplicate();
		sel2.moveToElementText(input);
		caretPos = -1;
		while(sel2.inRange(sel)){
			sel2.moveStart("character");
			caretPos++;
		}
	}
	// Firefox support
	else if (input.selectionStart || input.selectionStart == '0')
		caretPos = input.selectionStart;
	return (caretPos);
}


function q_setCaretToPos(input, pos) {
	if (input.setSelectionRange) {
		this.setSelectionRange(input, pos, pos);
	} else if (document.selection && input.createTextRange) {
		var range = input.createTextRange();
		range.collapse(true);
		range.moveEnd('character', pos);
		range.moveStart('character', pos);
		range.select();
	}  
}

/*******************************************************************************************/