/*

Copyright c 2013 Xyleme, Inc., 1881 9th Street, Suite 300, Boulder, Colorado 80302 USA.
All rights reserved.

This file and related documentation are protected by copyright and
are distributed under licenses regarding their use, copying, distribution,
and decompilation. No part of this product or related documentation may
be reproduced or transmitted in any form or by any means, electronic or
mechanical, for any purpose, without the express written permission of
Xyleme, Inc.

*/

// --------------------------------------------------
// JS functions for LedgerQuestion question
// --------------------------------------------------
//
// LedgerQuestion() : definition of Object type;
// getParamValue() : get value of specified question parameter;
// setParam() : set value for specified question parameter;
// redraw() : redraw the question according the current data stored in question object; 
// setUserResult() : store user answers;
// addLedgerQuestion() : add question to question array;
// getQuestion() : get question from question array;
// setLedgerQuestions() : set parameters for all questions in array;
//
// handleLedgerActivity() : evaluate user answer and invoke proper action;
// checkLedgerSubmit() : check if submit is valid;
// show() : show element of question form (image, feedback...);
// hide() : hide element of question form;
// setDisabled() : disable whole question;
// setAttemptLimit() : set number of attempts remaining;
// getHint() : get question hint if available;
// getUserResults() : update question object with any existing data present in CRE;
// setUserResults() : set user results to CRE;
// initLedgerPage() : init the CRE object;
//

scorableContent = true;   //Used by CRE to determine if the page is scorable
LedgerQ = true;               // defined in questions.js
var ledgerquestions = new Array();
var ledgerRowCount = 0;
var corrRowCount = 0;
var corrAnswArr = new Array();
var debitCreditArr = new Array();
var corrDebitCredit = 0;
var ledgerWidths = new Array();
var ledgerMap = new Object();

// ---------------------------------
// --- definition of Object type ---
function LedgerQuestion() {
	
    this.redraw = function () {
        //redraw the attempts left (as it goes from XSLT so the number is incorrect)
        var at = document.getElementById(this.guid + "_at_limit"); // change text node informing of remaining attempts

        // After ScoreCard (Course is finished)
        if (CRE && (typeof(CRE.getFinished) == "undefined") || CRE.getFinished()) {     // "undefined" when preview from scorecard 
            this.attempts = 0;
            this.showMarks = "true";

            setDisabled(this.guid);
            disableInputOnClick(this.guid);

            var submitBtn = document.getElementById(this.guid + "_submitBtn");
            if (submitBtn != null)
                submitBtn.disabled = "disabled";

            show(this.guid + "_advice");
            show(this.guid + "_feedback");

            if (at != null) at.firstChild.nodeValue = "(0 " + document.langAttemptRemaining + ")";
        }

        if (this.attempts == 1) {
            at.firstChild.nodeValue = "(" + this.attempts + " " + document.langAttemptRemaining + ")";
        } else {
            at.firstChild.nodeValue = "(" + this.attempts + " " + document.langAttemptsRemaining + ")";
        }
        var rowCount = parseInt(jQuery("#" + this.guid).attr("rowcount"));

        if (this.userAnswer == null || this.userAnswer == "") {
            return;
        }
        // set checked options
        var user_answerArr = this.userAnswer.split("^^^");

        for (var k = 0; k < rowCount; k++) {
            var elms = jQuery("[name=" + this.guid + "_row_" + (k + 1) + "_cell]");
            var user_answer = user_answerArr[k].split("|");
            for (var i = 0; i < elms.length; i++) {
                switch (elms[i].tagName.toUpperCase()) {
                    case "INPUT":
                        if (elms[i].getAttribute("formula") == "yes") {
                            elms[i].value = parseFloat(user_answer[i].substring(user_answer[i].indexOf("correct___") + 10, user_answer[i].length));
                        } else {
                            elms[i].value = user_answer[i];
                        }
                        if (typeof(elms[i].getAttribute("calculationFormula") != 'undefined' && elms[i].getAttribute("calculationFormula") != null)) {
                            ledgerMap[this.guid + "_" + elms[i].getAttribute("calculationFormula")] = parseFloat(elms[i].value);
                        }
                        var elmsParent = document.getElementById(elms[i].id + "_parentdiv");
                        if (elmsParent) {
                            if (user_answer[i] == "true") {
                                elms[i].setAttribute("src", img_path + "checkedRadioButton.png");
                                elms[i].checked = "checked";
                            } else {
                                elms[i].setAttribute("src", img_path + "uncheckedRadioButton.png");
                            }
                        }
                        break;
                    case "SELECT":
                        jQuery("#" + elms[i].id + " option[value='" + user_answer[i] + "']").attr("selected", "selected");
                        break;
                    case "IMG":
                        var elmsParent = document.getElementById(elms[i].id + "_parentdiv");
                        if (user_answer[i] == "true") {
                            elms[i].setAttribute("src", img_path + "checkedRadioButton.png");
                            elmsParent.style.backgroundColor = "#4EA391";
                            elmsParent.parentNode.style.backgroundColor = "#4EA391";
                        } else {
                            elms[i].setAttribute("src", img_path + "uncheckedRadioButton.png");
                            elmsParent.style.backgroundColor = "#ffffff";
                            elmsParent.parentNode.style.backgroundColor = "#ffffff";
                        }
                        break;
                }
            }
        }

        //evaluate question
       if (this.attempts == 0 || this.result == "correct") {
            this.attempts = this.attempts + 1; //increase attempts as handleLedgerActivity
            //will decrease it back when evaluating
            handleLedgerActivity(this.guid, "", true, rowCount, "true");
        }
    }
}

// Extend basic Question type
LedgerQuestion.prototype = new Question;

// --------------------------------------
// --- add question to question array ---
function addLedgerQuestion(question) {
    ledgerquestions[ledgerquestions.length] = question;
    addQuestion(question);
}

// -------------------------------------------------
// --- set parameters for all ledgerquestions in array ---
function setLedgerQuestions() {

    for (l = 0; l < document.forms.length; l++) {
        var gname = document.forms[l].name;
        if (gname.toLowerCase().indexOf("questionform") == -1) continue;
        var fid = document.forms[l].id;
        if (fid.toLowerCase().indexOf("ledgerquestion") == -1) continue;
        var debitCredit = document.forms[l].getAttribute("debitCredit");

        var wab_id = gname.substr(13);
        var wab_obj = document.getElementById("wab_" + wab_id);
        var newQuestion = new LedgerQuestion();
        newQuestion.setParam("guid", wab_id);
        newQuestion.setParam("attempts",
                wab_obj.getAttribute("wab_AttemptLimit"));
		newQuestion.setParam("showFeedback",		
				wab_obj.getAttribute("wab_ShowFeedback"));
        newQuestion.setParam("showHint",
                wab_obj.getAttribute("wab_ShowHint"));
		newQuestion.setParam("showAdvice",
                wab_obj.getAttribute("wab_ShowAdvice"));       
        newQuestion.setParam("hintActive",
                wab_obj.getAttribute("wab_hint_active"));
        newQuestion.setParam("correctAnswer",
                wab_obj.getAttribute("wab_correct_answer"));
		newQuestion.setParam("leaveCorrectBetweenAttempts", 
                wab_obj.getAttribute("wab_LeaveCorrectBetweenAttempts"));
        newQuestion.setParam("instantSubmit",
                wab_obj.getAttribute("wab_InstantSubmit"));
        newQuestion.setParam("showMarks", wab_obj.getAttribute("wab_ShowMarks"));
        newQuestion.setParam("debitCredit", debitCredit);
		newQuestion.setParam("overrideQuestionWeight", 
			wab_obj.getAttribute("wab_overridequestionweight"));	
		newQuestion.setParam("showOptionFeedback", wab_obj.getAttribute("wab_showoptionfeedback"));	       
        addLedgerQuestion(newQuestion);
        wab_obj.setAttribute("wab_correct_answer",encodeTxt(newQuestion.getParamValue("correctAnswer")));
        if (wab_obj.getAttribute("wab_ShowFeedback") == "true") {
            jQuery('#' + wab_id + '_feedback_pos').addClass('Show ShowThis');
        }
    }
}

function editCorrectAnswer(corrAnsw, pattern) {
    var correctAnswer = corrAnsw;
    var corrArr = correctAnswer.split("|")
    for (var j = 0; j < corrArr.length; j++) {
        if (corrArr[j].indexOf(pattern) != -1) {
            var v = corrArr[j].substring(0, corrArr[j].indexOf(pattern));
            var elms_value = parseFloat(v);
            if (isNaN(elms_value)) {
                elms_value = v;
            }
            corrArr[j] = elms_value;
        }
    }
    correctAnswer = "";
    for (var k = 0; k < corrArr.length - 1; k++) {
        correctAnswer += corrArr[k] + "|";
    }
    return correctAnswer;
}

// ---------------------------------------------------------
// --- handling submited form of LedgerQuestion question ---
function handleLedgerActivity(qid, msg, redraw, rowcount, feedback) {
    var submitBtn = document.getElementById(qid + "_submitBtn");
    var user_anw = String(checkLedgerSubmit(qid, msg, rowcount)).split("^^^");    // concat row answers with "^^^"

    var currentQuestion = getQuestion(qid);
    var at_limit = currentQuestion.getParamValue("attempts"); // attempt_limit
    var sfeedback = currentQuestion.getParamValue("showFeedback");
    var showAdvice = currentQuestion.getParamValue("showAdvice");
    var shint = currentQuestion.getParamValue("showHint");
    var instantSubmit = currentQuestion.getParamValue("instantSubmit");

    var leaveAnw = currentQuestion.getParamValue("leaveResponsesBetweenAttempts");
    var showMarks = currentQuestion.getParamValue("showMarks");
    var showOptionFeedback = currentQuestion.getParamValue("showOptionFeedback");
    var showScore = currentQuestion.getParamValue("showScore");
    var qWeight = currentQuestion.getParamValue("rollupWeight");
    var isCurrentLedgerRight = false;

    var allCorrectAnswers = currentQuestion.getParamValue("correctAnswer").split("^^^"); // get correct answers

    for (var i = 0; i < rowcount; i++) {
        var correctAnswer = allCorrectAnswers[i];
        var debitCreditAnswer = correctAnswer;
        var numberPattern = "_number";
        var debitCreditPattern = "_debitcredit";
        if (correctAnswer.indexOf(numberPattern) != -1) {
            correctAnswer = editCorrectAnswer(correctAnswer, numberPattern);
        }
        if (correctAnswer.indexOf(debitCreditPattern) != -1) {
            correctAnswer = editCorrectAnswer(correctAnswer, debitCreditPattern);
        }
        if (debitCreditAnswer.indexOf(debitCreditPattern) != -1) {
            var corrArr = debitCreditAnswer.split("|")
            for (var j = 0; j < corrArr.length; j++) {
                if (corrArr[j].indexOf(debitCreditPattern) != -1) {
                    var tempValue = corrArr[j];
                    corrArr[j] = corrArr[j + 1];
                    corrArr[j + 1] = tempValue;
                    break;
                }
            }
            debitCreditAnswer = "";
            for (var k = 0; k < corrArr.length - 1; k++) {
                debitCreditAnswer += corrArr[k] + "|";
            }
            debitCreditAnswer = editCorrectAnswer(debitCreditAnswer, debitCreditPattern);
            debitCreditArr.push(debitCreditAnswer.toLowerCase());
        }
        corrAnswArr.push(correctAnswer.toLowerCase());
    }

    for (var l = 0; l < rowcount; l++) {
        var checkmark = document.getElementById(qid + "_row_" + (l + 1) + "_answer");

        // ===== parsing for Formula ======
        var tempValue = "correct___";
        if (user_anw[l].indexOf(tempValue) != -1) {
            var arr = user_anw[l].split("|");
            for (var j = 0; j < arr.length; j++) {
                if (arr[j].indexOf(tempValue) != -1) {
                    arr[j] = arr[j].substring(0, arr[j].indexOf(tempValue) + 7);
                }
            }
            user_anw[l] = "";
            for (var k = 0; k < arr.length - 1; k++) {
                user_anw[l] += arr[k] + "|";
            }
        }
        // ================================

        for (i = 0; i < corrAnswArr.length; i++) {
            if (corrAnswArr[i] == user_anw[l]) {
                corrRowCount ++;
                isCurrentLedgerRight = true;
                break;
            }
        }

        if (debitCreditArr.length > 0) {
            for (i = 0; i < debitCreditArr.length; i++) {
                if (debitCreditArr[i] == user_anw[l]) {
                    corrDebitCredit ++;
                    debitCreditArr.splice(i, 1);
                    break;
                }
            }
        }

        for (i = 0; i < corrAnswArr.length; i++) {
            if (corrAnswArr[i] == user_anw[l]) {
                corrAnswArr.splice(i, 1);
                if (isCurrentLedgerRight || leaveAnw == "true" || at_limit == 1) {
                    if (sfeedback === ShowFeedback.all) {
                        checkmark.setAttribute("src", img_path + "correct_answer.png");
                    }
                    if ((instantSubmit != "true") || at_limit == 1) {
                        disableLedgerRow(qid + "_row_" + (l + 1));
                    }
                }
                break;
            } else {
                if (at_limit == 1) {
                   if (sfeedback === ShowFeedback.all) {
                        checkmark.setAttribute("src", img_path + "incorrect_answer.png");
                    }
                    if ((instantSubmit != "true") || at_limit == 0) {
                        disableLedgerRow(qid + "_row_" + (l + 1));
                    }
                }
            }
        }
    }

    if (corrRowCount == parseInt(rowcount)) {
        document.getElementById(qid + "_at_limit").style.visibility = "hidden";
        currentQuestion.setParam("result", "correct"); // set question result
        if (sfeedback === ShowFeedback.all) {
			fillImage(qid + "_feedback_q", "correct");
		}
    } else {
        currentQuestion.setParam("result", "incorrect"); // set question result
        if (sfeedback === ShowFeedback.all) {
			fillImage(qid + "_feedback_q", "incorrect");
		}
    }

    if (at_limit == 1 || (corrRowCount == parseInt(rowcount))) {

        setDisabled(qid); // mark question options disabled
        disableInputOnClick(qid);
        submitBtn.disabled = "disabled";

        if (showAdvice == "true" && feedback == "true") {
            show(qid + "_advice"); // show Advice
        }
    }
    if (corrDebitCredit == parseInt(rowcount) && sfeedback === ShowFeedback.all && at_limit == 1) {
        show(qid + "_debitcredit");
    }

	
	handleFeedback(qid, sfeedback, at_limit, corrRowCount == parseInt(rowcount)); // show Feedback

    corrAnswArr = new Array();
    debitCreditArr = new Array();
    corrRowCount = 0;
    corrDebitCredit = 0;

    setAttemptLimit(qid, at_limit);  // decrease attempts + update tailed text
    if (!redraw) {
        currentQuestion.setUserResult();//send result to CRE here!
        setUserResults();
    }

}

function disableLedgerRow(rowId) {
    var elms = jQuery("[name=" + rowId + "_cell]");
    for (var i = 0; i < elms.length; i++) {
        elms[i].disabled = "disabled";
    }
}

// -----------------------------
// --- check submited values ---
// qid = question id, msg = message to display if no option is selected
function checkLedgerSubmit(qid, msg, rowCount) {
    var user_answer = "";
    for (var k = 1; k <= parseInt(rowCount); k++) {
        var ledgerRowId = qid + "_row_" + k;
        var elms = jQuery("[name=" + ledgerRowId + "_cell]");

    for (var i = 0; i < elms.length; i++) {
        switch (elms[i].tagName) {
            case "INPUT":
				var element_value = jQuery.trim(elms[i].value);
                if(elms[i].getAttribute("numbers")== "yes"){					
                    var parsedNumber = parseNumber(element_value);
                    if(isNaN(parsedNumber)){ 
						parsedNumber = element_value;   // restore previous value if parsedNumber is not a correct number. e.g. contains some text
					}
                    user_answer += parsedNumber + "|";
                } else if (elms[i].getAttribute("formula") == "yes") {
                    calculateCellFormula(elms[i].id);
                    var expression = elms[i].getAttribute("expression");
                    var formula = new Formula();
                        var tempExpression = formulaExpression(expression, ledgerRowId, true);
                    var expressionValue = (formula.expressionEvaluate(tempExpression));
                    if (parseFloat(expressionValue) == parseNumber(element_value)) {
                        user_answer += "correct___" + element_value + "|";
					} else {
						user_answer += "incorrect___"+ element_value +"|"
					}
                } else if (elms[i].getAttribute("type") == "checkbox") {
					if (elms[i].checked) {
						user_answer += "true|";
					} else {
						user_answer += "false|";
					}
				} else if (elms[i].getAttribute("type") == "radio") {
					if (elms[i].checked) {
						user_answer += "true|";
					} else {
						user_answer += "false|";
					}
				}else {
					user_answer += element_value + "|";
				}
                break;
            case "SELECT":
                user_answer += jQuery.trim(elms[i].options[elms[i].selectedIndex].text) + "|";
                break;
            case "IMG":
                if (elms[i].getAttribute("src").indexOf("uncheckedRadioButton.png") == -1) {
                    user_answer += "true|";
                } else {
					user_answer += "false|";
				}
            break;
        }
    }
        user_answer += "^^^";
    }
    getQuestion(qid).setParam("userAnswer", user_answer);
    return user_answer.toLowerCase();
}

// -------------------------------------------------------
// --- reveal button functionality for Ledger question ---
function revealLedgerActivity(qid, rowCount, redraw) {
    var currentQuestion = getQuestion(qid);
    var allCorrectAnswers = currentQuestion.getParamValue("correctAnswer").split("^^^"); // get correct answers

    for (var k = 0; k < parseInt(rowCount); k++) {
        var correctAnswer = allCorrectAnswers[k];
        var numberPattern = "_number";
        var debitCreditPattern = "_debitcredit";
        if (correctAnswer.indexOf(numberPattern) != -1) {
            correctAnswer = editCorrectAnswer(correctAnswer, numberPattern);
        }
        if (correctAnswer.indexOf(debitCreditPattern) != -1) {
            correctAnswer = editCorrectAnswer(correctAnswer, debitCreditPattern);
        }
        var ledgerRowId = qid + "_row_" + (k + 1);
        var elms = jQuery("[name='" + ledgerRowId + "_cell" + "']");
        correctAnswer = correctAnswer.split("|");
        for (var i = 0; i < elms.length; i++) {
            var answer = jQuery.trim(correctAnswer[i]);
            switch (elms[i].tagName.toLowerCase()) {
                case "input":
                    if (elms[i].getAttribute("formula") == "yes") {
                        var expression = elms[i].getAttribute("expression");
                        var formula = new Formula();
                        var tempExpression = formulaExpression(expression, ledgerRowId, true);
                        var expressionValue = (formula.expressionEvaluate(tempExpression));
                        elms[i].value = expressionValue;
                        var parentDiv = jQuery("#div_" + elms[i].id);
                        if (parentDiv) {
                            parentDiv.html(expressionValue);
                        }
                    } else if (elms[i].getAttribute("type") == "checkbox") {
                        if (answer == "true") {
                            elms[i].checked = true;
                        } else {
                            elms[i].checked = false;
                        }
                    } else {
                        elms[i].value = answer;
                    }
                    break;
                case "select":
                    jQuery("#" + elms[i].id + " option[value='" + answer + "']").attr("selected", "selected");
                    break;
                case "img":
                    if (answer == "true") {
                        elms[i].setAttribute("src").indexOf("checkedRadioButton.png");
                    } else {
                        elms[i].setAttribute("src").indexOf("uncheckedRadioButton.png");
                    }
                    break;
            }
        }
    }

    setDisabled(qid); // mark question options disabled
    disableInputOnClick(qid);

    currentQuestion.setParam("attempts", 0);
    currentQuestion.setParam("userAnswer", "");
    if (!redraw) {
        setRevealStatus(qid);
        setUserResults();
    }

    show(qid + "_advice"); // show Advice
    var at = document.getElementById(qid + "_at_limit");
    at.style.visibility = "hidden";
    jQuery("#" + qid + "_revealBtn").attr("disabled", true);
    jQuery("#" + qid + "_submitBtn").attr("disabled", true);
    hide(qid + "_SubmitBox"); // hide Submin button
    hide(qid + "_RevealBox"); // hide Reveal button
}

// --- calculation of Formula  ---
function calculationFormula(qid, cellPos) {
    var questionId = qid.substring(0, qid.indexOf("_row_"));
    var formulas = jQuery("input[formulatype='text_" + questionId + "']");
    ledgerMap[questionId + "_" + cellPos] = parseFloat(jQuery("#" + qid).val());
    for (var i = 0; i < formulas.length; i++) {
        var formulaCell = jQuery(formulas[i]);
        var expression = formulaCell.attr("expression");
        var expressionTrimed = jQuery.trim(expression);
        if (expressionTrimed.indexOf(cellPos) != -1 || cellPos == "") {
            var varArray = parseExpression(expressionTrimed);
            for (var j = 0; j < varArray.length; j++) {
                var cellPosition = parseVariable(varArray[j]);
                var cellId = questionId + "_row_" + cellPosition[0] + "_blank_" + cellPosition[1];
                var cellValue = jQuery("#" + cellId).val();
                if(cellValue == "" || isNaN(cellValue)){
                    cellValue = "0";
                }
                expressionTrimed = expressionTrimed.replace("$" + varArray[j] + "$", cellValue);
            }
            var formula = new Formula();
            var expressionValue = (formula.expressionEvaluate(expressionTrimed));
            jQuery("#div_" + formulaCell.attr("id")).html(expressionValue);
            formulaCell.val(expressionValue);
        }
    }
}

// Math operations for Cells
function calculateCellFormula(qid) {
    var input = jQuery('#'+qid),
        expr = input.attr('data-expression', expr) || input.attr('expression');

    //save original expression
    input.attr('data-expression', expr);
    
    //if there are no rows and no cols in formula - only numbers
    if (expr.indexOf('C') == -1 && expr.indexOf('R') == -1) {
       displayResult(qid, expr);
       return;
    }
    // else

    // remove '=' sign
    if (expr.charAt(0) == '=') expr = expr.replace('=','');

    // if formula looks like - sum(C#R#:C#R#)
    expr = sumCells(expr);

    // replace C#R# with cell values
    expr = replaceIndexes(expr);
    input.attr('expression', expr);
    displayResult(qid, expr);

    if (input.attr('type') == 'hidden') { 
        input.attr('value', eval(expr))
    } else {
        input.attr('value', input.val());
    }

    // replace C#R# with cell values
    function replaceIndexes(expr) {
        while (expr.indexOf('C') != -1 && expr.indexOf('R') != -1) {
            var c_index = expr.indexOf('C'),
                r_index = expr.indexOf('R'),
                c_num   = expr.substring(c_index+1, r_index),
                r_num   =  0;

            for (var i = r_index+1, n = expr.length; i<= n; i++) {
              if (isNaN(expr.charAt(i)) || i == n) {
                r_num = expr.substring(r_index+1, i);
                break;
              }  
            }
            expr = expr.replace('C' + c_num + 'R' + r_num, getCellValue(c_num, r_num) );
        }
        return expr;    
    }


    // get value of cell[col,row] 
    function getCellValue(col, row) {
        var table = input.parents('.LedgerTable'),
            rows  = jQuery('tbody tr', table),
            res   = rows.eq(row).find('td:eq(' + (col-1) + ') > div > input').val();

        res = (res) ? res : rows.eq(row).find('td:eq(' + (col-1) + ') > span > input').val();
        if (!isNaN(res)) {return res} else {return 0}
    }

    // parse Formula -  sum(C#R#:C#R#)
    // e.g. sum(C1R1:C1R3) => C1R1+C1R2+C1R3
    function sumCells(expr) {
        if (expr.indexOf('sum') == 0) {
            var col_1 = expr.substring(expr.indexOf('C')+1,expr.indexOf('R')),
                col_2 = expr.substring(expr.lastIndexOf('C')+1,expr.lastIndexOf('R'));
                row_1 = expr.substring(expr.indexOf('R')+1,expr.indexOf(':')),
                row_2 = expr.substring(expr.lastIndexOf('R')+1,expr.length-1);
            
            expr = '';
            //column range
            if (col_1 == col_2 && row_1 != row_2) {
                if (row_1 < row_2) {
                    for (var i = row_1, n = row_2; i <= n; i++) {
                        expr += 'C' + col_1 + 'R' + i + '+'; 
                    }    
                } else {
                    for (var i = row_2, n = row_1; i <= n; i++) {
                        expr += 'C' + col_1 + 'R' + i + '+'; 
                    }
                }
            // row range
            } else if (col_1 != col_2 && row_1 == row_2) {
                if (col_1 < col_2) {
                    for (var i = col_1, n = col_2; i <= n; i++) {
                        expr += 'C' + i + 'R' + row_1 + '+'; 
                    }    
                } else {
                    for (var i = col_2, n = col_1; i <= n; i++) {
                        expr += 'C' + i + 'R' + row_1 + '+'; 
                    }
                }
            // if same 
            } else if (col_1 == col_2 && row_1 == row_2) {
                expr = 'C' + col_1 + 'R' + row_1 + '+';
            }

            // remove last '+'
            expr = expr.substring(0, expr.length-1);
        }
        return expr;
    }

    // show result in sibling div
    function displayResult(qid, expr) {
        try {
            expr = eval(expr);
        } catch(err) {
            console.warn(err);
        }
        jQuery('#div_'+qid).text(expr || '0');
    }
}

//recalculation of Formula Cells on Blank Cells keyup
jQuery(document).ready(function($) {
    $('input[calculationFormula]').bind('keyup', function(){
        $('input[formula=yes]').each(function(){
            calculateCellFormula($(this).attr('id'));
        });
    })
});


function parseExpression(expression) {
    var varArray = new Array();
    var expressionValue = expression;
    for (var i = 0; i < expressionValue.length; i++) {
        if (expressionValue.charAt(i) == '$') {
            for (var k = i + 1; k < expressionValue.length; k++) {
                if (expressionValue.charAt(k) == '$') {
                    var variable = expressionValue.substring(i + 1, k);
                    varArray.push(variable);
                    i += variable.length + 1;
                    break;
                }
            }
        }
    }
    return varArray;
}

function parseVariable(variable) {
    var array = new Array();
    var varValue = variable;
    array[0] = varValue.substring(1, varValue.indexOf("C"));
    array[1] = varValue.substring(varValue.indexOf("C") + 1, varValue.length);
    return array;
}

function initLedgerPage() {
    setLedgerQuestions();
    getUserResults(ledgerquestions);
}

function disableInputOnClick(qid) {
    var elms = document.getElementsByName(qid + "_cell");
    for (var i = 0; i < elms.length; i++) {
        elms[i].onclick = "";
        if(elms[i].tagName =="IMG"){
            elms[i].parentNode.onmousedown = "";
            elms[i].parentNode.style.cursor = "default";
        }
    }
}

function switchRadios(id , style) {
	var div = document.getElementById(id); 
	var imgid = id.substr(0,id.lastIndexOf("_parentdiv"));
	var img = document.getElementById(imgid);
	var elems = document.getElementsByName(id.substr(0, id.lastIndexOf("_cell")) + "_cell");
	var src = img.getAttribute("src");
	var choiceselectornumbergroup = id.substr(id.indexOf("_group_") + 7, 1);  // Group number limit 9

	if (src.indexOf("uncheckedRadioButton.png") == -1){
		img.setAttribute("src", img_path + "uncheckedRadioButton.png");
		if (style == "aat"){
			div.style.backgroundColor = "#ffffff";
			div.parentNode.style.backgroundColor = "#ffffff";
		}
	} else {
		for (var i = 0; i < elems.length; i++){
		    if (elems[i].tagName == "IMG") {
		        if (id.indexOf("_group_") != -1){
                    if (elems[i].id.substr(id.indexOf("_group_") + 7, 1) == choiceselectornumbergroup){
			            elems[i].setAttribute("src", img_path + "uncheckedRadioButton.png");
						if (style == "aat"){
							document.getElementById(elems[i].id + "_parentdiv").style.backgroundColor = "#ffffff";
							document.getElementById(elems[i].id + "_parentdiv").parentNode.style.backgroundColor = "#ffffff";
						}
		            }
                }else if (elems[i].id.indexOf("_group_") == -1) {
                    elems[i].setAttribute("src", img_path + "uncheckedRadioButton.png");
					if (style == "aat"){
						document.getElementById(elems[i].id + "_parentdiv").style.backgroundColor = "#ffffff";
						document.getElementById(elems[i].id + "_parentdiv").parentNode.style.backgroundColor = "#ffffff";
					}
                }
			}
		}
		img.setAttribute("src", img_path + "checkedRadioButton.png");
		if (style == "aat"){
			div.style.backgroundColor = "#4EA391";
			div.parentNode.style.backgroundColor = "#4EA391";
		}
	}
}

function switchRadioNumbers(id, event) {
    if(event.keyCode == 9) return false; 
    var qid = id.substr(0, id.lastIndexOf("_radionumber_"));
    var radionumbergroup = id.substr(id.indexOf("_group_") + 7, 1);  // Group number limit 9
    var elems = document.getElementsByName(qid + "_cell");

    for (var i = 0; i < elems.length; i++) {
        if ((elems[i].getAttribute("radionumber") == "rn") && (elems[i].id != id) && (elems[i].id.substr(elems[i].id.indexOf("_group_") + 7, 1) == radionumbergroup)) {
            elems[i].value = "";
        }
    }
}

function ledgerTableResize(qid) {
    var tableWidth = 0;
    jQuery("#" + qid + " tr:first td").each(function() {
        tableWidth += parseInt(jQuery(this).attr("width"));
    });
    if (jQuery("#" + qid).width() < tableWidth) {
        jQuery("#" + qid).width(tableWidth + 10);
    }
}