/*

Copyright c 2013 Xyleme, Inc., 1881 9th Street, Suite 300, Boulder, Colorado 80302 USA.
All rights reserved.

This file and related documentation are protected by copyright and
are distributed under licenses regarding their use, copying, distribution,
and decompilation. No part of this product or related documentation may
be reproduced or transmitted in any form or by any means, electronic or
mechanical, for any purpose, without the express written permission of
Xyleme, Inc.

*/

 
// --------------------------------------------------
// JS functions for WebMatrix question
// --------------------------------------------------
//
// MatrixQuestion() : definition of Object type;
//   getParamValue() : get value of specified question parameter;
//   setParam() : set value for specified question parameter;
//   redraw() : redraw the question according the current data stored in question object; 
//   setUserResult() : store user answers;
// addMatrixQuestion() : add question to question array;
// getQuestion() : get question from question array;
// setMatrixQuestions() : set parameters for all questions in array;
//
// handleWebMatrixActivity() : evaluate user answer and invoke proper action;
// checkMatrixSubmit() : check if submit is valid;
// initMatrixPage() : init the CRE object;

// --------- following functions moved to questions.js ----------
// show() : show element of question form (image, feedback...);
// hide() : hide element of question form;
// setDisabled() : disable whole question;
// setAttemptLimit() : set number of attempts remaining;
// getHint() : get question hint if available;
// getUserResults() : update question object with any existing data present in CRE;
// setUserResults() : set user results to CRE;
//


scorableContent = true;   //Used by CRE to determine if the page is scorable               // defined in questions.js
MatrixQ = true
var matrixQuestions = new Array();

// ---------------------------------
// --- definition of Object type ---
function MatrixQuestion()
{
    this.randomize = false;
    this.redraw = function () {
	        //redraw the attempts left (as it goes from XSLT so the number is incorrect)
	        var at = document.getElementById(this.guid + "_at_limit"); // change text node informing of remaining attempts
			
			if (this.attempts == 1) 
				at.firstChild.nodeValue = "(" + this.attempts + " " + document.langAttemptRemaining + ")";
			else  
				at.firstChild.nodeValue = "(" + this.attempts + " " + document.langAttemptsRemaining + ")";
	       
	        var elms = document.forms["QuestionForm_"+this.guid].elements;

			// set checked options
			if (this.attempts == 0 || this.result == "correct") {
				for (var i=0; i<elms.length; i++) {
					var answ = elms[i].value.split(this.guid +"_option_").join("");
					if (this.userAnswer.indexOf(answ) == -1 ) continue;
						elms[i].checked = true;
				}
				this.attempts = this.attempts + 1; //increase attempts as handleWebMatrixActivity 
												   //will decrease it back when evaluating
				handleWebMatrixActivity(this.guid,"","",true);
			}
        
		}
}

// Extend basic Question type
MatrixQuestion.prototype = new Question;

// --------------------------------------
// --- add question to question array ---
function addMatrixQuestion(question)
{
    matrixQuestions[matrixQuestions.length] = question;
    addQuestion(question);
}

// -------------------------------------------------
// --- set parameters for all questions in array ---
function setMatrixQuestions()
{
    for (l=0; l<document.forms.length; l++) {
        var fname = document.forms[l].name;     
        if (fname.toLowerCase().indexOf("questionform") == -1) continue;
        var fid = document.forms[l].id;
        if (fid.toLowerCase().indexOf("webmatrix") == -1) continue;

        var wab_id = fname.substr(13);
        
        var wab_obj = document.getElementById("wab_"+wab_id);
        newMatrixQuestion = new MatrixQuestion();
        newMatrixQuestion.setParam("guid", wab_id);
        var params = wab_obj.attributes;    
        var param;

        newMatrixQuestion.setParam("randomize", 
                wab_obj.getAttribute("data-wab_RandomizeChildren")); 
        newMatrixQuestion.setParam("attempts", 
                wab_obj.getAttribute("data-wab_AttemptLimit"));      
        newMatrixQuestion.setParam("showHint", 
                wab_obj.getAttribute("data-wab_ShowHint"));  
        newMatrixQuestion.setParam("showFeedback", 
                wab_obj.getAttribute("data-wab_ShowFeedback"));  
		newMatrixQuestion.setParam("showAdvice", 
                wab_obj.getAttribute("data-wab_showAdvice")); 				
        newMatrixQuestion.setParam("hintActive", 
                wab_obj.getAttribute("data-wab_hint_active"));       
        newMatrixQuestion.setParam("corectAnswer", 
                wab_obj.getAttribute("data-wab_correct_answer"));
        newMatrixQuestion.setParam("rollupWeight", 
                wab_obj.getAttribute("data-wab_RollupWeight"));
        newMatrixQuestion.setParam("leaveCorrectBetweenAttempts", 
                wab_obj.getAttribute("data-wab_LeaveCorrectBetweenAttempts"));

        addMatrixQuestion(newMatrixQuestion);
    }
}

// ------------------------------------------------------------
// --- handling submited form of WebMatrix question ---
function handleWebMatrixActivity(qid, msg, title, redraw)
{
	// ---imagemapquiz variables---
	var submitBtn = document.getElementById(qid + "_submitBtn");
	var doneBtn = document.getElementById(qid + "_doneBtn");
	// ---imagemapquiz variables---
	
    var submits = String(checkMatrixSubmit(qid, msg, title)); // id of selected option
        submits = submits.split("|");
    
    if (submits[0] != "false") { //some option WAS selected
                
        var currentQuestion = getQuestion(qid);
        var at_limit = currentQuestion.getParamValue("attempts"); // attempt_limit
        var anws = currentQuestion.getParamValue("corectAnswer"); // get correct answer(s)
        anws = anws.split("|"); // split them by "|" to the anws array
		
	
	var sfeedback = getFeedbackMode(currentQuestion);
	
	var showAdvice = currentQuestion.getParamValue("showAdvice");
	var shint = currentQuestion.getParamValue("showHint");
		  
		
		// find out, if answer is correct
		// take all submits and check them 1 by 1 against correct answers
		var qmatch = true; // flag for all answers correct

		for (var i=0; i < anws.length-1; i++) {
			var match = false;
			var anw = qid + "_option_" + anws[i]; // create answer id from question id
			for (var j=0; j < submits.length-1; j++) {
				if (submits[j] == anw) match = true; //submits[j] is correct answer
			}
			if (!match) qmatch = false; // flag for whole question - correct/incorrect
		}
		for (var i=0; i < submits.length-1; i++) {
			var match = false;
			for (var j=0; j < anws.length-1; j++) {
				var anw = qid + "_option_" + anws[j]; // create answer id from question id
				if (submits[i] == anw) match = true; //submits[i] is correct answer
			}
			if (!match) qmatch = false; // flag for whole question - correct/incorrect
		}

		if (qmatch == true) { /////////// CORRECT ANSWER!
			currentQuestion.setParam("result", "correct"); // set question result
			for (var i=0; i < submits.length-1; i++) {
				createTransparentFeedbacksAll(qid);
				if (sfeedback === ShowFeedback.all) {
					fillImage(submits[i],"correct");
					show(submits[i] + "_img_label","inline");  // show image in front of option 
				}
				var currentOptionsId  = submits[i].substring(0,submits[i].indexOf("_chbox_"));
				hide(currentOptionsId + "_feedback_ir"); 
				
				if (sfeedback === ShowFeedback.all) {
					show(currentOptionsId + "_feedback_cr"); 
					show(currentOptionsId + "_feedback"); 
				} 
			}
			setDisabled(qid); // mark question options disabled
			if (doneBtn != null) {
				submitBtn.style.visibility = "hidden";
				show(qid + "_doneBtn");
				doneBtn.disabled = false;
			}

			
		} else { ///////// INCORRECT ANSWER! (not all correct anws checked or one or more incorrect anws)
			
			//if not last attempt, show hints and question incorrect label
			//if last attempt show also correct answers (depending on feedback) and disable question

			currentQuestion.setParam("result", "incorrect"); // set question result

			if (at_limit > 1) {
				var leaveAnw = currentQuestion.getParamValue("leaveCorrectBetweenAttempts");
				for (var i=0; i < submits.length-1; i++) document.getElementById(submits[i]).checked = false;
				
				if (leaveAnw == "true") {
					for (var i=0; i < submits.length-1; i++) { //mark correct/incorrect submitted answers
						var iscorr = false
						for (var j=0; j < anws.length-1; j++) {
							var anw = qid + "_option_" + anws[j]; // create answer id from question id
							if (submits[i] == anw) {
								iscorr = true;
								document.getElementById(submits[i]).checked = true;
							}
						}
						if (iscorr) {
							createTransparentFeedbacksAll(qid);
							if (sfeedback === ShowFeedback.all) {
								fillImage(submits[i],"correct");
								show(submits[i] + "_img_label","inline");  // show image in front of option 
							}
						}
					}
				}						
				// list of hints e.g. (0.1.1)       
				var h = currentQuestion.getParamValue("hintActive");
				if ((shint == "true") && (h.match(/1/))) {
					var hint_id = getHint(qid,h);  // get id of randomly selected hint
					show(hint_id); // show this current hint
				} 
			} else {
				if (sfeedback === ShowFeedback.all) {
				//rows with correct and incorrect answers
					var correctRows = ""; 
					var incorrectRows = ""; 
					
					for (var i=0; i < submits.length-1; i++) { //mark correct/incorrect submitted answers
						var iscorr = false;
						var isLineCorr = true;
						for (var j=0; j < anws.length-1; j++) {
							var anw = qid + "_option_" + anws[j]; // create answer id from question id
							if (submits[i] == anw) {
								iscorr = true;
							}
						}
						if (iscorr) {
							createTransparentFeedbacksAll(qid);
							fillImage(submits[i],"correct");
							show(submits[i] + "_img_label","inline");  // show image in front of option 
							
							var currentOptionsId  = submits[i].substring(0,submits[i].indexOf("_chbox_"));
							correctRows += currentOptionsId +"|"; 
							var incElms = incorrectRows.split("|")
							
							for (var k=0; k < incElms.length-1; k++) {
								if (currentOptionsId == incElms[k]) {
									isLineCorr = false;
									break;
								}
							}
							
							if (isLineCorr) {
								hide(currentOptionsId + "_feedback_ir"); 
								show(currentOptionsId + "_feedback_cr");
							}
							show(currentOptionsId + "_feedback"); 
						}
						else  {
							createTransparentFeedbacksAll(qid);
							fillImage(submits[i],"incorrect");
							show(submits[i] + "_img_label","inline");  // show image in front of option 
							
							var currentOptionsId  = submits[i].substring(0,submits[i].indexOf("_chbox_"));
							incorrectRows += currentOptionsId +"|"; 
							show(currentOptionsId + "_feedback_ir"); 
							hide(currentOptionsId + "_feedback_cr"); 
							show(currentOptionsId + "_feedback"); 
						}
					}
				
					
					for (var j=0; j < anws.length-1; j++) { // mark 'also correct' answers
						var isalsocorr = true;
						var anw = qid + "_option_" + anws[j]; // create answer id from question id
						for (var i=0; i < submits.length-1; i++) {
							if (submits[i] == anw) {
							
								isalsocorr = false;
							}
						}
						if (isalsocorr) {
							createTransparentFeedbacksAll(qid);
							fillImage(anw,"also_correct_backward");
							show(anw + "_img_label","inline");  // show image in front of option 
							
							var currentOptionsId  = anw.substring(0,anw.indexOf("_chbox_"));
							
							var cElms = correctRows.split("|");
							var incElms = incorrectRows.split("|");
							
							show(currentOptionsId + "_feedback_ir"); 
							
							for (var i=0; i< cElms.length-1; i++) {
								if (currentOptionsId == cElms[i]) {
									hide(currentOptionsId + "_feedback_ir"); 
									show(currentOptionsId + "_feedback_nq"); 
								}
							}
							
							for (var i=0; i< incElms.length-1; i++) {
								if (currentOptionsId == incElms[i]) {
									hide(currentOptionsId + "_feedback_nq"); 
									show(currentOptionsId + "_feedback_ir"); 
								}
							}
							
							hide(currentOptionsId + "_feedback_cr"); 
							show(currentOptionsId + "_feedback"); 
						}
					}
				} else { //last attempt, feedback = false
					forceIconsCleaning (qid);
				}				
				setDisabled(qid); // mark question options disabled
				if (doneBtn != null) {
					submitBtn.style.visibility = "hidden";
					show(qid + "_doneBtn");
					doneBtn.disabled = false;
				}
			}
		}
		
		handleAdvice(qid, showAdvice, at_limit, qmatch); // show Advice
		handleFeedback(qid, sfeedback, at_limit, qmatch); // show Feedback

		
        
		setAttemptLimit(qid,at_limit);  // decrease attempts + update tailed text
        if (!redraw) {
            currentQuestion.setUserResult();//send result to CRE here!
            setUserResults();
        }
    } // end of some option was selected
}
        


// -----------------------------
// --- check submited values ---
// qid = question id, msg = message to display if no option is selected
function checkMatrixSubmit(qid, msg, title)
{
    var test = false;
    var user_answer = "";
	var elms = document.getElementsByTagName('Input');
    
    for (var i=0; i < elms.length; i++) {
		var idToCheck = elms[i].value.substring(0,elms[i].value.indexOf("_option_"));
		if (elms[i].name.indexOf("matrix_chbox_" + qid)>=0 && elms[i].type == "radio" || elms[i].type == "checkbox" && qid == idToCheck){
		
			if (elms[i].checked != "") {
				test = true;
				user_answer += elms[i].value + "|";
			}
		}
    }
    if (!test) {
        if(msg) jAlert(msg);
        return test;
    } else {
        getQuestion(qid).setParam("userAnswer", user_answer);
        return user_answer;
    }
}

function createTransparentFeedbacksAll (qid) {
	var elms = document.getElementsByTagName('Input');
    for (var i=0; i < elms.length; i++) {
		var idToCheck = elms[i].value.substring(0,elms[i].value.indexOf("_option_"));
	
		if ((elms[i].type == "radio" || elms[i].type == "checkbox") && qid == idToCheck){
			var img = document.getElementById(elms[i].value + "_img");

			if (img != null && img.innerHTML.length < 7) {
				img.innerHTML = "<img  style=\"vertical-align: middle;\" src=\"" + img_path + "img_transp.png\" id=\"" + elms[i].value.concat("_img_label") + "\"/>"
				fillImage(elms[i].value,"transparent");
				show(elms[i].value + "_img_label","inline");
			}
		}
    }
}

//--- erases all icons ---
function forceIconsCleaning(qid) {

	var elms = document.getElementsByTagName('Input');
    
    for (var i=0; i < elms.length; i++) {
		var idToCheck = elms[i].value.substring(0,elms[i].value.indexOf("_option_"));
		var currentColumnId = elms[i].value.substring(elms[i].value.indexOf("_chbox_"),elms[i].value.length);
		
		if ((elms[i].type == "radio" || elms[i].type == "checkbox") && qid == idToCheck){
			var img = document.getElementById(elms[i].value + "_img");
			
			if (img != null) {
				img.innerHTML = "";
				fillImage(elms[i].value,"transparent");
				show(elms[i].value + "_img_label","inline");
			}
		}
    }
}
// -------------------------------
// --- init the whole CRE page ---

function initMatrixPage()
{   
    randomMQ(matrixQuestions[matrixQuestions.length-1]);
    setMatrixQuestions();
    getUserResults(matrixQuestions);
}

//window.onload = initQuestionPage();

function randomMQ(quest){
	if (quest==null)
		return;
	var rand = quest.getParamValue("randomize");
	if (rand != "true")
		return;	
	var id = 'WebMatrix_' + quest.getParamValue("guid");    
	var form = jQuery("#"+id);
	if (form == null)
		return;
		
	var question = qHTML = [];
	if (getArrangement(quest.guid) == null){
		jQuery("#"+id+" div[class='Option']").shuffle();
	}
	
	qHTML=jQuery("#"+id+" div[class='Option']");

	if (getArrangement(quest.guid) == null){
		jQuery("#"+id+" div[class='Option']").each(function(i, selected){ 
			question[i] = jQuery(selected).attr("id"); 
		});
		setArrangement(quest.guid, [question, [""]]);
	} else {
		var rez = getArrangement(quest.guid);
		question = rez[0];
		var q = new Array();
		jQuery("#"+id+" div[class='Option']").each(function(i, sel){
			for (var ii=0; ii < question.length; ii++){
				if (question[i] == jQuery(qHTML[ii]).attr("id"))
					q.push(jQuery(qHTML[ii]).html());
			}
		});
		jQuery("#"+id+" div[class='Option']").each(function(i, sel){jQuery(sel).html(q[i]);});
	}
	/* correct background-color after shuffle*/
	jQuery("#"+id+" div.Option:even").each(function(){	       
			var col1 = jQuery(this).children('div:first')			
			if(col1.hasClass('Light')) col1.removeClass('Light').addClass('Dark')						
	})

	jQuery("#"+id+" div.Option:odd").each(function(){	       
			var col1 = jQuery(this).children('div:first')			
			if(col1.hasClass('Dark')) col1.removeClass('Dark').addClass('Light')						
	})		
	/* end correct background-color*/	
	
}