/*

Copyright c 2013 Xyleme, Inc., 1881 9th Street, Suite 300, Boulder, Colorado 80302 USA.
All rights reserved.

This file and related documentation are protected by copyright and
are distributed under licenses regarding their use, copying, distribution,
and decompilation. No part of this product or related documentation may
be reproduced or transmitted in any form or by any means, electronic or
mechanical, for any purpose, without the express written permission of
Xyleme, Inc.

*/

 
// --------------------------------------------------
// JS functions for SelectHotText question
// --------------------------------------------------
//
// SHTQuestion() : definition of Object type;
//   getParamValue() : get value of specified question parameter;
//   setParam() : set value for specified question parameter;
//   redraw() : redraw the question according the current data stored in question object; 
//   setUserResult() : store user answers;
// addSHTQuestion() : add question to question array;
// getQuestion() : get question from question array;
// setSHTQuestions() : set parameters for all questions in array;
//
// handleSHTActivity() : evaluate user answer and invoke proper action;
// checkSHTSubmit() : check if submit is valid;
// initSHTPage() : init the CRE object;

// --------- following functions moved to questions.js ----------
// show() : show element of question form (image, feedback...);
// hide() : hide element of question form;
// setDisabled() : disable whole question;
// setAttemptLimit() : set number of attempts remaining;
// getHint() : get question hint if available;
// getUserResults() : update question object with any existing data present in CRE;
// setUserResults() : set user results to CRE;
//


scorableContent = true;   //Used by CRE to determine if the page is scorable
SHTQ = true;               // defined in questions.js
var shtquestions = new Array();

if (!Array.prototype.indexOf)
{
  Array.prototype.indexOf = function(elt /*, from*/)
  {
    var len = this.length >>> 0;

    var from = Number(arguments[1]) || 0;
    from = (from < 0)
         ? Math.ceil(from)
         : Math.floor(from);
    if (from < 0)
      from += len;

    for (; from < len; from++)
    {
      if (from in this &&
          this[from] === elt)
        return from;
    }
    return -1;
  };
}

// ---------------------------------
// --- definition of Object type ---
function SHTQuestion() {
    this.redraw = function () {
            //redraw the attempts left (as it goes from XSLT so the number is incorrect)
            var at = document.getElementById(this.guid + "_at_limit"); // change text node informing of remaining attempts
		
			if (this.attempts == 1) {
				at.firstChild.nodeValue = "(" + this.attempts + " " + document.langAttemptRemaining + ")";
			} else  {
				at.firstChild.nodeValue = "(" + this.attempts + " " + document.langAttemptsRemaining + ")";
			}
			           
			var user_anws = String(this.userAnswer).split("|");
			var cor_anws = String(this.corectAnswer).split("|");
			var handleOfQuestion = this;
			jQuery('#SelectHotText_' + this.guid + ' .TextOptionChoice').each(function (i, elem) {
				if (handleOfQuestion.attempts == 0) {
					attachFeedback(jQuery(elem).attr('id'), handleOfQuestion);
					jQuery(elem).attr('onclick','');
				}
				if (user_anws.indexOf(jQuery(this).attr('id')) != -1) {
					if (cor_anws.indexOf(jQuery(elem).attr('id')) != -1) {
						jQuery(elem).removeClass('UnansweredText').addClass('CorrectedAnsweredText').attr("data-checked", "true");
					} else {
						jQuery(elem).removeClass('UnansweredText').addClass('IncorrectedSelectText').attr("data-checked", "true");
					}
				}
			});
			
            //evaluate question
            //if question is done, add +1 attempt and call handleSHTActivity 
            //  to render the question result. Attempts are decreased in this function by 1   
            //if question is not done yet (there are some spare attempts)
            //  ommitted (AND all hints which were shown before)
            if (this.attempts == 0 || this.result == "correct") {
                this.attempts = this.attempts + 1; //increase attempts as handleSHTActivity 
                                                   //will decrease it back when evaluating
                handleSHTActivity(this.guid,"","",true);
            }            
        }
}

// Extend basic Question type
SHTQuestion.prototype = new Question;

// --------------------------------------
// --- add question to question array ---
function addSHTQuestion(question) {
    shtquestions[shtquestions.length] = question;
    addQuestion(question);
}

// TODO move "returnAttr" function to question.js when all questions WAB attributes will be updated with "data-" prefix
// this function also present in WebDndSequenceActivity.js
var SHTQUtils = { 
	returnAttr : function(selected, obj){
					var attributePrefix = 'data-';
					if (obj.indexOf(attributePrefix) != -1){
						attributePrefix = '';
					}
					var attr = jQuery(selected).attr(attributePrefix + obj);
					return (typeof attr=='undefined' || attr=='undefined') ? null : attr;
				}
}

// -------------------------------------------------
// --- set parameters for all questions in array ---
function setSHTQuestions() {
    jQuery(".SelectHotText form[id*='SelectHotText'] input.WebActivityBehavior").each(function(i, selected){
	
		var qid = jQuery(selected).attr('id').replace('wab_','');
		
        var newSHTQuestion = new SHTQuestion();
        newSHTQuestion.setParam("guid", qid);
        newSHTQuestion.setParam("attempts", SHTQUtils.returnAttr(selected, "wab_AttemptLimit"));
        newSHTQuestion.setParam("showHint", SHTQUtils.returnAttr(selected, "wab_ShowHint"));
        newSHTQuestion.setParam("showFeedback", SHTQUtils.returnAttr(selected, "wab_ShowFeedback"));
        newSHTQuestion.setParam("showAdvice", SHTQUtils.returnAttr(selected, "wab_showAdvice"));
        newSHTQuestion.setParam("hintActive", SHTQUtils.returnAttr(selected, "wab_hint_active"));
        newSHTQuestion.setParam("corectAnswer", Base64.decode(SHTQUtils.returnAttr(selected, "wab_correct_answer")));        
        newSHTQuestion.setParam("rollupWeight", SHTQUtils.returnAttr(selected, "wab_RollupWeight"));
        newSHTQuestion.setParam("leaveCorrectBetweenAttempts", SHTQUtils.returnAttr(selected, "wab_LeaveCorrectBetweenAttempts"));

        addSHTQuestion(newSHTQuestion);
    });
}

// ------------------------------------------------------------
// --- handling submited form of WebMultipleChoice question ---
function handleSHTActivity(qid, msg, title,redraw) {
	// ---imagemapquiz variables---
	var submitBtn = document.getElementById(qid + "_submitBtn");
	var doneBtn = document.getElementById(qid + "_doneBtn");
	// ---imagemapquiz variables---
	
    var submits = String(checkSHTSubmit(qid, msg, title)); // id of selected option
    submits = submits.split("|");
    
    if (submits[0] != "false") { //some option WAS selected
        var currentQuestion = getQuestion(qid);
        var at_limit = currentQuestion.getParamValue("attempts"); // attempt_limit
        var anws = currentQuestion.getParamValue("corectAnswer"); // get correct answer(s)
        anws = anws.split("|"); // split them by "|" to the anws array
		
		var sfeedback = getFeedbackMode(currentQuestion);
		
		var showAdvice = currentQuestion.getParamValue("showAdvice");
	  
		var qmatch = true; // flag for all answers correct
		
		// find out, if answer is correct
		// take all submits and check them 1 by 1 against correct answers

		for (var i=0; i < anws.length-1; i++) {
			var match = false;
			var anw = anws[i]; // create answer id from question id
			for (var j=0; j < submits.length-1; j++) {
				if (submits[j] == anw) {
					match = true; //submits[j] is correct answer
				}
			}
			if (!match) {
				qmatch = false; // flag for whole question - correct/incorrect
			}
		}
		for (var i=0; i < submits.length-1; i++) {
			var match = false;
			for (var j=0; j < anws.length-1; j++) {
				var anw = anws[j]; // create answer id from question id
				if (submits[i] == anw) {
					match = true; //submits[i] is correct answer
				}
			}
			if (!match) {
				qmatch = false; // flag for whole question - correct/incorrect
			}
		}

		if (qmatch == true) { /////////// CORRECT ANSWER!
			currentQuestion.setParam("result", "correct"); // set question result
			if (sfeedback === ShowFeedback.all) {
				for (var i = 0; i < submits.length-1; i++) {
					show(submits[i] + "_feedback_cr"); // show correct label
					show(submits[i] + "_feedback", "", true); // show feedback, if available
					jQuery('#' + submits[i]).addClass('CorrectedAnsweredText');
				}
				jQuery('#SelectHotText_' + qid + ' .TextOptionChoice').attr('onclick','');
				attachFeedbacks('#SelectHotText_' + qid + ' .TextOptionChoice', currentQuestion);
			}
			setDisabled(qid); // mark question options disabled
			if (doneBtn != null) {
				submitBtn.style.visibility = "hidden";
				show(qid + "_doneBtn");
				doneBtn.disabled = false;
			}
		} else { ///////// INCORRECT ANSWER! (not all correct anws checked or one or more incorrect anws)
			
			//if last attempt show also correct answers (depending on feedback) and disable question

			currentQuestion.setParam("result", "incorrect"); // set question result

			if (at_limit > 1) {
				var leaveAnw = currentQuestion.getParamValue("leaveCorrectBetweenAttempts");
				
				jQuery('#SelectHotText_' + qid +' .TextOptionChoice').removeClass('AnsweredText').addClass('UnansweredText').attr("data-checked", "");
				
				if (leaveAnw == "true") {
					jQuery('#SelectHotText_' + qid +' .TextOptionChoice').each(function () {
						var user_anws = String(currentQuestion.userAnswer).split("|");
						var cor_anws = String(currentQuestion.corectAnswer).split("|");
						if (user_anws.indexOf(jQuery(this).attr('id')) != -1 && cor_anws.indexOf(jQuery(this).attr('id')) != -1) {
							jQuery(this).removeClass('UnansweredText').addClass('CorrectedSelectText AnsweredText').attr("data-checked", "true").attr('onclick','').unbind('click');
						}
					});
				}
			} else {	
				jQuery('#SelectHotText_' + qid + ' .TextOptionChoice').attr('onclick','');
				if (sfeedback === ShowFeedback.all) {
					attachFeedbacks('#SelectHotText_' + qid + ' .TextOptionChoice', currentQuestion);
					for (var i=0; i < submits.length-1; i++) { //mark correct/incorrect submitted answers
						var iscorr = false
						for (var j=0; j < anws.length-1; j++) {
							var anw = anws[j]; // create answer id from question id
							if (submits[i] == anw) iscorr = true;
						}
						if (iscorr) {
							jQuery('#' + submits[i]).addClass('CorrectedAnsweredText');
						} else  {
							//attachFeedback(submits[i]);
							jQuery('#' + submits[i]).addClass('IncorrectedSelectText');
							jQuery('#' + submits[i]).trigger('click'); // show incorrect label
						}
						show(submits[i] + "_feedback", "", true); // show response feedback
					}
					for (var j=0; j < anws.length-1; j++) { // mark 'also correct' answers
						var isalsocorr = true;
						var anw = anws[j]; // create answer id from question id
						for (var i=0; i < submits.length-1; i++) {
							if (submits[i] == anw) isalsocorr = false
						}
						if (isalsocorr) {
							show(anw + "_feedback_tr"); // show 'This is the...' text
							show(anw + "_feedback", "", true); // show response feedback
							jQuery('#' + anws[j]).addClass('CorrectedUnSelectText');
						}
					}
				}
				setDisabled(qid); // mark question options disabled
				if (doneBtn != null) {
					submitBtn.style.visibility = "hidden";
					show(qid + "_doneBtn");
					doneBtn.disabled = false;
				}
			}
		}
        
		
		handleAdvice(qid, showAdvice, at_limit, qmatch); // show Advice 
		handleHint(qid, currentQuestion, at_limit, qmatch); // show Hint
		handleFeedback(qid, sfeedback, at_limit, qmatch); // show Feedback
		
        setAttemptLimit(qid,at_limit);  // decrease attempts + update tailed text
        if (!redraw) {
            currentQuestion.setUserResult();//send result to CRE here!
            setUserResults();
        }
    } // end of some option was selected
}
        
// -----------------------------
// --- check submited values ---
// qid = question id, msg = message to display if no option is selected
function checkSHTSubmit(qid, msg, title) {
    var test = false;
    var user_answer = "";

	jQuery('#SelectHotText_' + qid +' .TextOptionChoice').each(function () {
		if (jQuery(this).attr("data-checked") == "true") {
            test = true;
            user_answer += jQuery(this).attr('id') + "|";
        }
	});
    if (!test) {
        if(msg) jAlert(msg, title);
        return test;
    } else {
        getQuestion(qid).setParam("userAnswer", user_answer);
        return user_answer;
    }
}

// -------------------------------
// --- init the whole CRE page ---

function initSHTPage() {  
    setSHTQuestions();
    getUserResults(shtquestions);
}

function attachFeedback(elem, handleOfQuestion) {
	if (getRequestParameter("mode") == "review" || handleOfQuestion.getParamValue("showFeedback") == "true"){
		var dialog = jQuery(jQuery('#' + elem).find('.SHTFeedback')).dialog({
			autoOpen: false,
			show: "fade",
			hide: "fade",
			title: hide,
			autoResize:true,
			minHeight: 26
		});
		jQuery('#' + elem).click(function(){
			jQuery(dialog).find('.ui-dialog-titlebar').hide();
			var cur = jQuery(this);
			jQuery(dialog).dialog("option", { position: [(cur.position().left + cur.width() / 2), (cur.position().top) + cur.height() - jQuery(document).scrollTop()]});
			jQuery(dialog).dialog('open');
		});
	}
}
function attachFeedbacks(id, handleOfQuestion) {
	jQuery(id).each(function (i, elem) {
		attachFeedback(jQuery(elem).attr('id'), handleOfQuestion);
	});
}

function chooseText(current, qid){
	if (getQuestion(qid).getParamValue("corectAnswer").split("|").length < 3){
		jQuery('#SelectHotText_' + qid +' .TextOptionChoice').not(current).removeClass('AnsweredText').addClass('UnansweredText').attr("data-checked", "");
	}
	if (jQuery(current).attr("data-checked") == "true") {		
		jQuery(current).removeClass('AnsweredText').addClass('UnansweredText').attr("data-checked", "");
	} else {
		jQuery(current).removeClass('UnansweredText').addClass('AnsweredText').attr("data-checked", "true");
	}
}