/*

Copyright c 2013 Xyleme, Inc., 1881 9th Street, Suite 300, Boulder, Colorado 80302 USA.
All rights reserved.

This file and related documentation are protected by copyright and
are distributed under licenses regarding their use, copying, distribution,
and decompilation. No part of this product or related documentation may
be reproduced or transmitted in any form or by any means, electronic or
mechanical, for any purpose, without the express written permission of
Xyleme, Inc.

*/

 
// --------------------------------------------------
// JS functions for WebFillInBlank question
// --------------------------------------------------
//
// FBQuestion() : definition of Object type;
//   getParamValue() : get value of specified question parameter;
//   setParam() : set value for specified question parameter;
//   redraw() : redraw the question according the current data stored in question object; 
//   setUserResult() : store user answers;
// addFBQuestion() : add question to question array;
// getQuestion() : get question from question array;
// setFBQuestions() : set parameters for all questions in array;
//
// handleWebFibActivity() : evaluate user answer and invoke proper action;
// checkFIBSubmit() : check if submit is valid;
// show() : show element of question form (image, feedback...);
// hide() : hide element of question form;
// setDisabled() : disable whole question;
// setAttemptLimit() : set number of attempts remaining;
// getHint() : get question hint if available;
// getUserResults() : update question object with any existing data present in CRE;
// setUserResults() : set user results to CRE;
// initPage() : init the CRE object;
//

// lines marked with xxx are commented/uncommented depending on onCRE/offCRE environment


scorableContent = true;   //Used by CRE to determine if the page is scorable
FBQ = true;               // defined in questions.js
var fbquestions = new Array();

// ---------------------------------
// --- definition of Object type ---
function FBQuestion() {
    this.redraw = function () {
	
	if(this.userAnswer == null) {
		return;
	}
			
	//redraw the attempts left (as it goes from XSLT so the number is incorrect)
		var at = document.getElementById(this.guid + "_at_limit"); // change text node informing of remaining attempts
		
		if (this.attempts == 1) at.firstChild.nodeValue = "(" + this.attempts + " " + document.langAttemptRemaining + ")";
		else  at.firstChild.nodeValue = "(" + this.attempts + " " + document.langAttemptsRemaining + ")";
					
		//set (fill in) user answer		
		var blanks;
	
		if (isIE){
			blanks = getElementsByName("input", this.guid+"_blank");
			var blanksS = getElementsByName("select", this.guid+"_blank");		
			blanks = blanks.concat(blanksS);
		} else {
			blanks = document.getElementsByName(this.guid+"_blank");
		}
		
		var user_anws;

		if (getRequestParameter("mode") == "review" 
				&& navigator.appName != "Microsoft Internet Explorer" ) {
			user_anws = (utf8Decode(unescape(this.userAnswer))).split("||");
		} else {
			user_anws = unescape(this.userAnswer).split("||");
		}

         if (blanks != null) {
             for (var i = 0; i < blanks.length; i++) {
                 for (var j = 0; j < user_anws.length; j++) {
                     var anw = user_anws[j].toString().split(":::");

                     if (blanks[i].id == this.guid + anw[0]) {
                         switch (blanks[i].tagName.toLowerCase()) {
                            case "input":
                                if (typeof(blanks[i].getAttribute("expression")) != 'undefined' && blanks[i].getAttribute("expression") != null) {
                                    blanks[i].value = anw[1].substring(anw[1].indexOf("___")+3,anw[1].length); // check if formula
                                } else {
									blanks[i].value = anw[1];
								}
                            break;
                            case "select":
                                jQuery("#" + blanks[i].id + " option[value='" + anw[1] + "']").attr("selected", "selected");
							break;
                         }
                     }
                 }
             }
         }	 

		//evaluate question
		if (this.attempts == 0 || this.result == "correct") {
			this.attempts = this.attempts + 1; //increase attempts as handleWebFibActivity 
											   //will decrease it back when evaluating
			handleWebFibActivity(this.guid,"","",true);
		}
	}
}

// Extend basic Question type
FBQuestion.prototype = new Question;

// --------------------------------------
// --- add question to question array ---
function addFBQuestion(question) {
    fbquestions[fbquestions.length] = question;
    addQuestion(question);
}

// -------------------------------------------------
// --- set parameters for all fbquestions in array ---
function setFBQuestions() {
    for (l=0; l<document.forms.length; l++) {
        var fname = document.forms[l].name;     
        if (fname.toLowerCase().indexOf("questionform") == -1) continue;
        var fid = document.forms[l].id;
        if (fid.toLowerCase().indexOf("webfillinblank") == -1) continue;

        var wab_id = fname.substr(13);
		var blanks;
	
		if (isIE){
			blanks = getElementsByName("input", wab_id+"_blank");
			var blanksS = getElementsByName("select", wab_id+"_blank");		
			blanks = blanks.concat(blanksS);
		} else {
			blanks = document.getElementsByName(wab_id+"_blank");
		}
		
		for (var i=0; i<blanks.length; i++) {
			blanks[i].onkeypress = stopRKey;
		}

        var wab_obj = document.getElementById("wab_"+wab_id);
        var newQuestion = new FBQuestion();
        newQuestion.setParam("guid", wab_id);

		newQuestion.setParam("attempts", 
			wab_obj.getAttribute("data-wab_AttemptLimit"));	
		newQuestion.setParam("showHint", 
			wab_obj.getAttribute("data-wab_ShowHint"));	
		newQuestion.setParam("showFeedback", 
			wab_obj.getAttribute("data-wab_ShowFeedback"));	
		newQuestion.setParam("showAdvice",
			wab_obj.getAttribute("data-wab_ShowAdvice"));
		newQuestion.setParam("hintActive", 
			wab_obj.getAttribute("data-wab_hint_active"));	
		newQuestion.setParam("correctAnswer", 
			decodeCorrAnswer(wab_obj.getAttribute("data-wab_correct_answer"))); //decode correct answer
		newQuestion.setParam("rollupWeight", 
		   wab_obj.getAttribute("data-wab_RollupWeight"));
		newQuestion.setParam("leaveCorrectBetweenAttempts", 
		   wab_obj.getAttribute("data-wab_LeaveCorrectBetweenAttempts"));
		addFBQuestion(newQuestion);
    }
}

function isValueFromInterval(tolerance , value, correct){
    if (value <= correct + tolerance && value >= correct - tolerance) {
        return true;
    }
    return false;
}

// ---------------------------------------------------------
// --- handling submited form of WebFillInBlank question ---
function handleWebFibActivity(qid, msg, title, redraw, arr) {
	// ---imagemapquiz variables---
	var submitBtn = document.getElementById(qid + "_submitBtn");
	var doneBtn = document.getElementById(qid + "_doneBtn");
	// ---imagemapquiz variables---

	var blanks;
	
	if (isIE){
		blanks = getElementsByName("input", qid+"_blank");
		var blanksS = getElementsByName("select", qid+"_blank");		
		blanks = blanks.concat(blanksS);
	} else {
		blanks = document.getElementsByName(qid+"_blank");
	}
	 
    var filled = checkFIBSubmit(qid,blanks,msg, title, arr);

    if (filled) {
		var currentQuestion = getQuestion(qid);
		
		var sfeedback = getFeedbackMode(currentQuestion);
		
		var correct_answer = currentQuestion.getParamValue("correctAnswer");
		var user_answer = currentQuestion.getParamValue("userAnswer");
		var at_limit = currentQuestion.getParamValue("attempts"); // get current attempt_limit
		var showAdvice = currentQuestion.getParamValue("showAdvice");
		var ev_blanks = new Array(); //stores false/true for incorrect/correct blank
		var correct;
		var qmatch = true;
		
		correct_answer = correct_answer.split("||");
		user_answer = user_answer.split("||");
		for (var j=0; j < blanks.length; j++) {// check all blank inputs
			var answer = correct_answer[j].split("|"); // multiple correct answers are possible
			ev_blanks[j] = true;
			correct = false;
            var answ = user_answer[j].toString().split(":::");
            for (var i = 0; i < answer.length; i++) {
				var correct_answer_trimmed = jQuery.trim(answer[i]);				
				
                switch (blanks[j].tagName.toLowerCase()) {
                    case "input":
						var blank_value = jQuery.trim(blanks[j].value);
						if (typeof(blanks[j].getAttribute("min")) != 'undefined' && blanks[j].getAttribute("min") != null) {
		                    var parsedNumber = parseNumber(blank_value);
		                    if(!isNaN(parsedNumber)) {
								blank_value = parsedNumber;   // use parsed value if it is correct number
							}
						}
                        var tolerance = blanks[j].getAttribute("tolerance");
                            
                        if (typeof(blanks[j].getAttribute("expression")) != 'undefined' && blanks[j].getAttribute("expression") != null) {
                            if (answ[1].indexOf("___") != -1) {
                                var temp = answ[1].substring(0, answ[1].indexOf("___"))
                                if (temp == correct_answer_trimmed) {
                                    correct = true;
                                }
                            }
                        } else {							
                            if (!isNaN(blank_value) && !isNaN(correct_answer_trimmed)) {
                                if(blank_value != "") {
                                    blank_value = parseFloat(blank_value);
                                }
                                correct_answer_trimmed = parseFloat(correct_answer_trimmed);
                            }
                            if (typeof(tolerance) != 'undefined' && tolerance != null) {
                                correct = isValueFromInterval(parseFloat(tolerance), blank_value, correct_answer_trimmed);
                            } else {
								var min = blanks[j].getAttribute("min");
								var max = blanks[j].getAttribute("max");
								if (min && max && blank_value >= min && blank_value <= max) {
									correct = true;
								}
                            }
							if (blanks[j].getAttribute("date") == "true"){
								var d1 = new String(answer).split("-");
								var d2 = new String(blank_value).split("-");
								var dr = 0;
								for (var i=0; i<3; i++){
									if (parseInt(d1[i], 10) == parseInt(d2[i], 10)){
										dr++;
									}
								}
								correct = dr == 3;
							}
                        }
                        break;
                    case "select":
						var option_value = jQuery.trim(blanks[j].options[blanks[j].selectedIndex].innerHTML);						
                        if (!isNaN(option_value) && !isNaN(correct_answer_trimmed)) {
                            if(option_value!=""){
                                option_value = parseFloat(option_value);
                            }
                            correct_answer_trimmed = parseFloat(correct_answer_trimmed);
                        }
                        if (option_value == correct_answer_trimmed) {
                            correct = true;
                        }
                        break;
                }
            }
			if (!correct) {
			qmatch = false;
			ev_blanks[j] = false;
			}
		}
        //text matches
        var textMatchAnswer = checkTextMatches(blanks);
        for (var i = 0; i < textMatchAnswer.length; i++) {
            ev_blanks[textMatchAnswer[i] - 1] = true;
        }
        (jQuery.inArray(false, ev_blanks)!= -1) ? qmatch = false : qmatch = true;

		if (qmatch) {
			///// CORRECT ANSWER!
			// mark blanks as correct, show feedback if True and disable question
			
			currentQuestion.setParam("result", "correct");   
			
			if (sfeedback === ShowFeedback.all) {
				for (var i=0; i < blanks.length; i++) {
					fillImage(blanks[i].id,"correct");  // first replace image on blank 
					show(blanks[i].id + "_img_label","inline");  // show image 
				}
			}
			setDisabled(qid); // disable question
			jQuery("input[name = " + qid + "_date" + "]").each(function(index, element) {
				jQuery(element).attr("disabled", true);
			});
			if (doneBtn != null) {
				hide(qid + "_submitBtn");
				show(qid + "_doneBtn");
				doneBtn.disabled = false;
			}
		} else {
			///// INCORRECT ANSWER! (not all correct anws checked or one or more incorrect anws)
			// if last attempt and feedback is true, show also correct answers and disable question
			
			currentQuestion.setParam("result", "incorrect");   
			
			if (at_limit > 1) { //this is not last attempt, so show hints, if available
				var leaveAnw = currentQuestion.getParamValue("leaveCorrectBetweenAttempts");

				if (leaveAnw == "true" && sfeedback === ShowFeedback.all) {
					for (var j=0; j < blanks.length; j++) { // check all blank inputs
						if (ev_blanks[j]) {
							fillImage(blanks[j].id,"correct");                           // replace image on blank 
							show(blanks[j].id + "_img_label","inline");                  // show image on blank
						} else {
							fillImage(blanks[j].id,"transparent");                           // replace image on blank 
							show(blanks[j].id + "_img_label","inline");                  // show image on blank
							document.getElementById(blanks[j].id).value = ''; //clear blanks
						}
					}
				} else {
					for (var i=0; i < blanks.length; i++) document.getElementById(blanks[i].id).value = ''; //clear blanks
				}				
			} else { // it was last attemp, unfortunately incorrect answer
				if (sfeedback === ShowFeedback.all) {
					for (var i=0; i < blanks.length; i++) {
						if (ev_blanks[i]) {
							fillImage(blanks[i].id,"correct");                           // replace image on blank 
							show(blanks[i].id + "_img_label","inline");  // show image 
						} else {
							fillImage(blanks[i].id,"incorrect");                           // replace image on blank 
							show(blanks[i].id + "_img_label","inline");  // show image 
						}
					}
				}
				setDisabled(qid); // disable question 
				jQuery("input[name = " + qid + "_date" + "]").each(function(index, element) {
					jQuery(element).attr("disabled", true);
				});
				if (doneBtn != null) {
					hide(qid + "_submitBtn")
					show(qid + "_doneBtn","");
					doneBtn.disabled = false;
				}
			}
		}
		
		handleAdvice(qid, showAdvice, at_limit, qmatch); // show Advice
		handleHint(qid, currentQuestion, at_limit, qmatch); // show Hint
		handleFeedback(qid, sfeedback, at_limit, qmatch); // show Feedback
		
		setAttemptLimit(qid,at_limit);  // decrease attempts + update tailed text
		if (!redraw) {
			currentQuestion.setUserResult();// send result to CRE here!
			setUserResults();
		}
    }
}
		


// -----------------------
// --- fill user entry ---
function fillUserEntry(blank_id) {
    var input = document.getElementById(blank_id);
    var entry = document.getElementById(blank_id + "_feedback_user");

    entry.firstChild.nodeValue = input.value + ": ";
}


// ---------------------------
// --- fill correct answer ---
function fillCorrAnswer(blank_id) {
    var anws = document.getElementsByName(blank_id + "_anw");
    var anw;
    for (var i=0; i < anws.length; i++) {
		if(anws[i].firstChild!= null) {
			anw = anws[i].firstChild.nodeValue;
			anws[i].firstChild.nodeValue = decodeText(String(anw));
		}
    }
}


// -----------------------------
// --- check submited values ---
// qid = question id, blank = blank input to check, 
function checkFIBSubmit(qid, blanks, msg, title, arr) {
    var user_anw = "";
    var filled = true;
	var currentQuestion = getQuestion(qid);
	var instantSubmit = currentQuestion.getParamValue("instantSubmit");
	
	// check for empty blanks
    for (var i = 0; i < blanks.length; i++) {
        switch (blanks[i].tagName.toLowerCase()) {
            case "input":
                if (jQuery.trim(blanks[i].value) == '') {
					filled = false;
				}
				
				if (typeof(blanks[i].getAttribute("charactersSize")) != 'undefined' && blanks[i].getAttribute("charactersSize") != null) {
					var charactersSize = blanks[i].getAttribute("charactersSize");
					if (blanks[i].value.length < charactersSize) {
						filled = false;
					}
				}
            break;
            case "select":
                if (jQuery.trim(jQuery(blanks[i]).val()) == '') {
                    filled = false;
                }
            break;
        }
    }

    if (filled == true) {
        for (var j = 0; j < blanks.length; j++) {
            var blanksValue = "";
            switch (blanks[j].tagName.toLowerCase()) {
                case "input":
                    var expression = blanks[j].getAttribute("expression");
                    var tolerance = blanks[j].getAttribute("tolerance");
                    if (typeof(expression) != 'undefined' && expression != null) {
                        // check formula value
                        var correctValue = "correct___" + blanks[j].value;
                        var incorrectValue = "incorrect___" + blanks[j].value; 
                        var formula = new Formula();
                        var tempExpression = formulaExpression(expression, qid, false, arr);
                        var expressionValue = (formula.expressionEvaluate(tempExpression));
                        if (typeof(tolerance) != 'undefined' && tolerance != null) {
                            if(isValueFromInterval(parseFloat(tolerance), parseNumber(blanks[j].value), parseFloat(expressionValue))){
                                blanksValue = correctValue;
                            } else {
                                blanksValue = incorrectValue;
                            }
                        } else if (parseFloat(expressionValue) == parseNumber(blanks[j].value)) {
                            blanksValue = correctValue;
                        } else {
							blanksValue = incorrectValue;
						}
                    } else {
						blanksValue = blanks[j].value;
					}
                break;
                case "select":
                    blanksValue = jQuery(blanks[j]).val();
                break;
            }
            user_anw += blanks[j].id.split(qid + "_blank_").join("_blank_") + ":::" + blanksValue + "||";
        }
		getQuestion(qid).setParam("userAnswer", user_anw);
		return true;
	} else { 
		if(msg){
			jAlert(msg, title); // show "Check an option please" message box
		}
		return false;
	}
}

function checkTextMatches(blanks) {
    var blankNumberArr = new Array();
    for (var j = 0; j < blanks.length; j++) {
        if (blanks[j].getAttribute("textmatches") == "true") {
            var userAnswer = blanks[j].value;
            var indexCount = 0;
            var blankId = blanks[j].id;
            var blankNumber = blankId.substring(blankId.indexOf("_blank_")+7, blankId.length);
            var textMatchesId = "TextMatch_" + blankId;
            var textMatches = jQuery("#" + textMatchesId);
            var matchAtLeastTM = parseInt(textMatches.attr("matchatleast"));
            var textMatchArr = textMatches.children();
            for (var i = 0; i < textMatchArr.length; i++) {
                var matchAtLeastPattern = parseInt(jQuery(textMatchArr[i]).attr("matchatleast"));
                var patternArr = jQuery(textMatchArr[i]).children();
                if (checkPatterns(patternArr, userAnswer, matchAtLeastPattern)) { 
                    indexCount ++;
                }
                if(indexCount == matchAtLeastTM){
                    blankNumberArr.push(parseInt(blankNumber));
                    break;
                }
            }
        }
    }
    return blankNumberArr;
}

function checkPatterns(patternArr, userAnswer, matchAtLeast) {

    var indexCount = 0;
    for (var k = 0; k < patternArr.length; k++) { // for Pattern
        var answer = userAnswer;
        var pattern = jQuery(patternArr[k]);
        var patternText = Base64.decode(pattern.attr("patterntext"));
        var isIgnoreCase = pattern.attr("ignorecase");
        var isIgnoreSpaces = pattern.attr("ignorespaces");
        var isIgnoreHyphensAndMinus = pattern.attr("ignorehyphensandminus");
        var isIgnoreExtraWords = pattern.attr("ignoreextrawords");
        if (isIgnoreCase == "true") {
            answer = answer.toLowerCase();
            patternText = patternText.toLowerCase();
        }
        if (isIgnoreHyphensAndMinus == "true") {
            answer = answer.replace(/-/gi, "");
            patternText = patternText.replace(/-/gi, "");
        }
        var userAnswerWithSpaces = jQuery.trim(answer);
        if (isIgnoreSpaces == "true") {
            answer = answer.replace(/\s/g, "");
            patternText = patternText.replace(/\s/g, "");
        }
        
        patternText = patternText.replace(/\?/g,".?");
        patternText = patternText.replace(/\*/g,".*");
        var re = new RegExp("^" + patternText + "$");
        if (isIgnoreExtraWords == "true") {
            var splitUserAnswer = userAnswerWithSpaces.split(" ");
            for(i = 0; i < splitUserAnswer.length; i++){
                if (re.test(splitUserAnswer[i]) == true) {
                    indexCount++;
                }
            }
        } else {
            //if (re.test(answer) == true) {
            answer = answer.split(',');
            for (var i = 0, n = answer.length; i < n; i++) {
            	answer[i] = jQuery.trim(answer[i]);
            	if (answer[i] == patternText) {
                	indexCount++;
            	}	
            }
        }
        if (indexCount == matchAtLeast) {
            return true;
        }
    }
    return false;
}

// -----------------------------
// --- decode correct answer ---
function decodeCorrAnswer(answer) {
    var blanks_decoded = new String;
    var anws_decoded = new String;
    var anws = new Array;

    var blanks_anw = answer.split("||");

    for (var i=0; i < blanks_anw.length; i++) {
	anws_decoded = '';
	anws = blanks_anw[i].split("|");
	for (var j=0; j < anws.length; j++) {
	    if (j > 0) anws_decoded += "|"; 
	    anws_decoded += decodeText(anws[j]);
	}
	if (i > 0) blanks_decoded += "||"; 
	blanks_decoded += anws_decoded;	
    }
    return blanks_decoded;
}

function stopRKey(evt) { 
  var evt = (evt) ? evt : ((event) ? event : null); 
  var node = (evt.target) ? evt.target : ((evt.srcElement) ? evt.srcElement : null); 
  if ((evt.keyCode == 13) && (node.type=="text"))  {return false;} 
}

// -------------------------------
// --- init the whole CRE page ---

function initFBPage() {   
    setFBQuestions();
    getUserResults(fbquestions);
}