/*

Copyright c 2013 Xyleme, Inc., 1881 9th Street, Suite 300, Boulder, Colorado 80302 USA.
All rights reserved.

This file and related documentation are protected by copyright and
are distributed under licenses regarding their use, copying, distribution,
and decompilation. No part of this product or related documentation may
be reproduced or transmitted in any form or by any means, electronic or
mechanical, for any purpose, without the express written permission of
Xyleme, Inc.

*/

 
// --------------------------------------------------
// JS functions for WebGridQuestion question
// --------------------------------------------------
//
// GridQuestion() : definition of Object type;
// getParamValue() : get value of specified question parameter;
// setParam() : set value for specified question parameter;
// redraw() : redraw the question according the current data stored in question object; 
// setUserResult() : store user answers;
// addGridQuestion() : add question to question array;
// getQuestion() : get question from question array;
// setGridQuestions() : set parameters for all questions in array;
//
// handleWebGridActivity() : evaluate user answer and invoke proper action;
// checkGridSubmit() : check if submit is valid;
// show() : show element of question form (image, feedback...);
// hide() : hide element of question form;
// setDisabled() : disable whole question;
// setAttemptLimit() : set number of attempts remaining;
// getHint() : get question hint if available;
// getUserResults() : update question object with any existing data present in CRE;
// setUserResults() : set user results to CRE;
// initPage() : init the CRE object;
//

// lines marked with xxx are commented/uncommented depending on onCRE/offCRE environment


scorableContent = true;   //Used by CRE to determine if the page is scorable
GridQ = true;               // defined in questions.js
var gridquestions = new Array();

// ---------------------------------
// --- definition of Object type ---
function GridQuestion()
{
}

// Extend basic Question type
GridQuestion.prototype = new Question;

// --------------------------------------
// --- add question to question array ---
function addGridQuestion(question)
{
    gridquestions[gridquestions.length] = question;
    addQuestion(question);
}

// -------------------------------------------------
// --- set parameters for all gridquestions in array ---
function setGridQuestions()
{
    for (l=0; l<document.forms.length; l++) {
        var gname = document.forms[l].name;     
        if (gname.toLowerCase().indexOf("questionform") == -1) continue;
        var fid = document.forms[l].id;
        if (fid.toLowerCase().indexOf("webgridquestion") == -1) continue;

        var wab_id = gname.substr(13);

        var wab_obj = document.getElementById("wab_"+wab_id);
        newQuestion = new GridQuestion();
        newQuestion.setParam("guid", wab_id);
        var params = wab_obj.attributes;    
        var param;

	newQuestion.setParam("attempts", 
		wab_obj.getAttribute("wab_AttemptLimit"));	
	newQuestion.setParam("showHint", 
		wab_obj.getAttribute("wab_ShowHint"));	
	newQuestion.setParam("showFeedback", 
		wab_obj.getAttribute("wab_ShowFeedback"));	
	newQuestion.setParam("hintActive", 
		wab_obj.getAttribute("wab_hint_active"));	
	newQuestion.setParam("correctAnswer", 
		decodeCorrAnswer(wab_obj.getAttribute("wab_correct_answer"))); //decode correct answer
    newQuestion.setParam("rollupWeight", 
       wab_obj.getAttribute("wab_RollupWeight"));
	newQuestion.setParam("leaveCorrectBetweenAttempts", 
       wab_obj.getAttribute("wab_LeaveCorrectBetweenAttempts"));
        addGridQuestion(newQuestion);
    }
}

// ---------------------------------------------------------
// --- handling submited form of WebGridQuestion question ---
function handleWebGridActivity(qid,msg,title,colNum)
{
	var submit = String(checkGridSubmit(colNum, qid, msg, title)); // id of selected option
	
	hide(qid + "_feedback_cr");
	hide(qid + "_feedback_pos");
	hide(qid + "_feedback_ir");
	hide(qid + "_feedback_neg");
	
	if (submit != "false") {
		
		var currentQuestion = getQuestion(qid);
		
		var userAnswer = currentQuestion.getParamValue("userAnswer");
        var anws = currentQuestion.getParamValue("correctAnswer"); // get correct answer(s)
		
		//Correct answer!
        var aaa = parseFloat(userAnswer);
        var bbb = parseFloat(anws);

		if (aaa == bbb) {
			show(qid + "_feedback_cr");
			show(qid + "_feedback_pos");
		} else {//NOT correct answer!
			show(qid + "_feedback_ir");
			show(qid + "_feedback_neg");
		}
	}
}

// -----------------------------
// --- check submited values ---
// qid = question id, input to check, 
function checkGridSubmit(colNum, qid, msg, title)
{
	var user_answer = new Array(colNum);
	var elms = document.getElementsByTagName('Input');
	var index;
	var answer = "";
	
	for (var j = 0; j < colNum; j++){
		user_answer[j] = -1;
	}

    for  (var i = 0; i < elms.length; i++){
        var idToCheck = elms[i].id.substring(0, elms[i].id.indexOf("_answer_"));
        if (qid == idToCheck){
            if (elms[i].value.match(/^\d{1}$/)){
                user_answer[i] = elms[i].value;
            }
        }
    }

	for (var j = 0; j < colNum; j++){
		if (user_answer[j] != -1){
			answer += user_answer[j];
		}
	}
	
	if (answer != ""){
		getQuestion(qid).setParam("userAnswer", answer);
		return true;
	} else {
		jAlert(msg, title);
		return false;
	}
}

// -------------------------------
// --- init the whole CRE page ---

function initGridQPage()
{
    setGridQuestions();
    getUserResults(gridquestions);
}

function ShowGridAnswer(qid, colNumber, rowNumber){
	document.getElementById(qid + "_answer_" + colNumber).value=rowNumber;
    for (var i = 0; i < 10; i++){
        document.getElementById(qid + '_gridQuestionNum_' + i + colNumber).style.backgroundImage = "url('" + img_path + "gridCircleBlack.png')";
    }
    document.getElementById(qid + '_gridQuestionNum_' + rowNumber + colNumber).style.backgroundImage = "url('" + img_path + "gridCircleBlue.png')";
}


function MarkTypedNumber (qid, colNumber, msg, title){
    if (document.getElementById(qid + '_answer_' + colNumber).value.match(/^\d{1}$/)){
        for (var i = 0; i < 10; i++){
            document.getElementById(qid + '_gridQuestionNum_' + i + colNumber).style.backgroundImage = "url('" + img_path + "gridCircleBlack.png')";
        }
        document.getElementById(qid + '_gridQuestionNum_' + document.getElementById(qid + '_answer_' + colNumber).value + colNumber).style.backgroundImage = "url('" + img_path + "gridCircleBlue.png')";
    }else {
        jAlert(msg, title);
    }
}