/*

Copyright c 2013 Xyleme, Inc., 1881 9th Street, Suite 300, Boulder, Colorado 80302 USA.
All rights reserved.

This file and related documentation are protected by copyright and
are distributed under licenses regarding their use, copying, distribution,
and decompilation. No part of this product or related documentation may
be reproduced or transmitted in any form or by any means, electronic or
mechanical, for any purpose, without the express written permission of
Xyleme, Inc.

*/


// --------------------------------------------------
// JS functions for WebMatchingActivity question
// --------------------------------------------------
//
// MAQuestion() : definition of Object type;
//   getParamValue() : get value of specified question parameter;
//   setParam() : set value for specified question parameter;
//   redraw() : redraw the question according the current data stored in question object; 
//   setUserResult() : store user answers;
// addMAQuestion() : add question to question array;
// getQuestion() : get question from question array;
// setMAQuestions() : set parameters for all questions in array;
//
// handleWebMatchingActivity() : evaluate user answer and invoke proper action;
// markMAOption() : handles complex marking of options as correct, incorrect...;
// checkMASubmit() : check if submit is valid;
// initMAPage() : init the CRE object;

// ------------- moved to questions.js ----------------------
// show() : show element of question form (image, feedback...);
// hide() : hide element of question form;
// setDisabled() : disable whole question;
// setAttemptLimit() : set number of attempts remaining;
// getHint() : get question hint if available;
// getUserResults() : update question object with any existing data present in CRE;
// setUserResults() : set user results to CRE;
//

// lines marked with xxx are commented/uncommented depending on onCRE/offCRE environment


scorableContent = true;   //Used by CRE to determine if the page is scorable
MAQ = true;               // defined in questions.js
var maquestions = new Array();
var random = true; //for call randomMAQ function when redraw only once 

// ---------------------------------
// --- definition of Object type ---
function MAQuestion() {
    this.randomize = false;
    this.redraw = function () {
	    //redraw the attempts left (as it goes from XSLT so the number is incorrect)
	    var at = document.getElementById(this.guid + "_at_limit"); // change text node informing of remaining attempts
		
		if (this.attempts == 1) {
			at.firstChild.nodeValue = "(" + this.attempts + " " + document.langAttemptRemaining + ")";
		} else {
			at.firstChild.nodeValue = "(" + this.attempts + " " + document.langAttemptsRemaining + ")";
		}

	    var user_anws = String(this.userAnswer).split("|");
	    if (this.attempts == 0 || this.result == "correct") {
			randomMAQ(this.guid);
			var mainDiv = jQuery('#MA_'+this.guid);
			var arr = new Array();
			
			for (var i=0; i<user_anws.length-1; i++) {
				var anw = user_anws[i].split(":");
				var distractor = jQuery('#'+this.guid+'_distractor_'+anw[1]+'_DistractorContent').parents('td').filter(':first');
				var match = jQuery('#'+this.guid+'_match_'+anw[0]+'_MatchContent').parents('td').filter(':first');
				if (distractor.length == 0 || match.length == 0)
					continue;
				var x1 = distractor.width()/2;
				var x2 = match.width()/2 +(match.offset().left - mainDiv.offset().left);
				var y1 = distractor.height()/2 + (distractor.offset().top - mainDiv.offset().top);
				var y2 = match.height()/2 + (match.offset().top - mainDiv.offset().top);
				arr[i]  = new jsGraphics(document.getElementById("MA_"+this.guid)); 
				arr[i].setStroke("2");
				arr[i].setColor("#000");
				arr[i].drawLine(x1,y1,x2,y2);
				arr[i].setColor("#054CA3");
				drawArrows(x1, y1, x2, y2, arr[i], "4");
				arr[i].paint();
				match.find('input').val(anw[1]);
			}
			//evaluate question
			this.attempts = this.attempts + 1; //increase attempts as handleWebMatchingActivity 
		                                   //will decrease it back when evaluating
			handleWebMatchingActivity(this.guid,"","",true);
	    }
	}
}
// Extend basic Question type
MAQuestion.prototype = new Question;

// --------------------------------------
// --- add question to question array ---
function addMAQuestion(question){
    maquestions[maquestions.length] = question;
    addQuestion(question);
}

// -------------------------------------------------
// --- set parameters for all questions in array ---
function setMAQuestions() {
    for (l=0; l<document.forms.length; l++) {
        var fname = document.forms[l].name;     
        if (fname.toLowerCase().indexOf("questionform") == -1) continue;
        var fid = document.forms[l].id;
        if (fid.toLowerCase().indexOf("webmatchingactivity") == -1) continue;

        var wab_id = fname.substr(13);

        var wab_obj = document.getElementById("wab_"+wab_id);
        newMAQuestion = new MAQuestion();
        newMAQuestion.setParam("guid", wab_id);

		newMAQuestion.setParam("randomize", 
			wab_obj.getAttribute("data-wab_RandomizeChildren"));	
		newMAQuestion.setParam("attempts", 
			wab_obj.getAttribute("data-wab_AttemptLimit"));	
		newMAQuestion.setParam("showHint", 
			wab_obj.getAttribute("data-wab_ShowHint"));	
		newMAQuestion.setParam("showFeedback", 
			wab_obj.getAttribute("data-wab_ShowFeedback"));	
		newMAQuestion.setParam("showAdvice", 
                wab_obj.getAttribute("data-wab_showAdvice"));	
		newMAQuestion.setParam("hintActive", 
			wab_obj.getAttribute("data-wab_hint_active"));	
		newMAQuestion.setParam("corectAnswer", 
			wab_obj.getAttribute("data-wab_correct_answer"));	
		newMAQuestion.setParam("rollupWeight", 
			wab_obj.getAttribute("data-wab_RollupWeight"));
		newMAQuestion.setParam("leaveCorrectBetweenAttempts", 
        wab_obj.getAttribute("data-wab_LeaveCorrectBetweenAttempts"));
        addMAQuestion(newMAQuestion);
		
    }
}

// --------------------------------------------------------------
// --- handling submited form of WebMatchingActivity question ---
function handleWebMatchingActivity(qid, msg, title, redraw, arr) {
	// ---imagemapquiz variables---
	var submitBtn = document.getElementById(qid + "_submitBtn");
	var doneBtn = document.getElementById(qid + "_doneBtn");
	// ---imagemapquiz variables---

    var submit = String(checkMASubmit(qid, msg, title)); // id of selected option
    submits = submit.split("|");
    
    if (submits[0] != "false") { //some option WAS selected
		var currentQuestion = getQuestion(qid);
		var at_limit = currentQuestion.getParamValue("attempts"); // attempt_limit
		var answer = currentQuestion.getParamValue("corectAnswer"); // get correct answer(s)
		var anws = answer.split("|");
		var showAdvice = currentQuestion.getParamValue("showAdvice");
		var showMarks = currentQuestion.getParamValue("showMarks");
		var showOptionFeedback = currentQuestion.getParamValue("showOptionFeedback");
		
		var sfeedback = getFeedbackMode(currentQuestion);

		// find out, if answer is correct
		var qmatch = true;
		for (var i=0; i < submits.length-1; i++) {
			if (answer.indexOf(submits[i]) == -1) qmatch = false; //submits[i] is incorrect answer
			//correct answers doesn't contain such pair 'match:distractor'
		}	

	if (qmatch == true) {
	///// CORRECT ANSWER!
	   //mark options as correct, show feedback if True and disable question
	    
	    currentQuestion.setParam("result", "correct"); // set question result

		var imageTds = getElementsByName("td", "MAansAnsw_" + qid);
		for (var i=0; i<imageTds.length; i++) {
			if (isIE){
				imageTds[i].style.display = "block"; 
			}else{
				imageTds[i].style.display = "table-cell"; 
			}
		}
		if (sfeedback === ShowFeedback.all) {
			for (var i=0; i < submits.length-1; i++) {
				var subm = submits[i].split(":");
				fillImage(qid + "_match_" + subm[0],"correct");
				show(qid + "_match_" + subm[0] + "_img_label","inline");  // show OK mark on option 
			}
		}
			setDisabled(qid); // disable question
			
			// remove events
			removeEvent(window, 'load', randomMAQ);
			jQuery("#WebMatchingActivity_" + qid).find("td").unbind();
						
			if (doneBtn != null) {
				submitBtn.style.visibility = "hidden";
				show(qid + "_doneBtn");
				doneBtn.disabled = false;
			}
		} else { 
		///// INCORRECT ANSWER! (not all correct anws checked or one or more incorrect anws)
			//if last attempt and feedback is true, show also correct answers and disable question

			currentQuestion.setParam("result", "incorrect"); // set question result

			if (at_limit > 1) {
				var leaveAnw = currentQuestion.getParamValue("leaveCorrectBetweenAttempts");
				var className = jQuery('#MA_'+qid).find("td[id = 'MAque1']").attr("class")
				var index = className.match(/from\d+/g);
				if (index != null){
							index = index.toString().substring(4);							
						}
				if(leaveAnw == "true"){
				  for (var i=0; i<submits.length-1; i++) 					
					  if (answer.indexOf(submits[i]) == -1) {
							for (var j=0;j<arr.length-1;j++)
								if(submits[i].split(":")[1] == arr[j+1].from) {
									arr[j+1].draw.clear();  //clear incorect answers
									arr[j+1].to = 0;
									jQuery('#MA_'+qid).find('#' + qid + '_match_' + submits[i].split(":")[0])
										.parent()
										.parent()
										.addClass('to'+index)
										.find('div.MatchSelect input')
										.val("")									
									break;
								}
					  } 				
				}else{
					for(var i=0;i<arr.length-1;i++){ 
				             arr[i+1].draw.clear();  //clear all answers
					}
					//reset all .to pointers (make all matchers selectable)
					jQuery("*[id*='" + qid + "_distractor']").parent().each(function(){						
							jQuery(this).next().addClass("to"+index);						
					});	
					for(var i=0;i<arr.length-1;i++){
						arr[i+1].to = 0;		
					}
					jQuery("input[name='" + qid + "_select']").attr("value","");
				}
			} else {
				// last attempt
				// disable question
				if (sfeedback === ShowFeedback.all) {
					for (var i=0; i<submits.length-1; i++) {
						var subm = qid + "_match_" + submits[i].split(":")[0];
						if (answer.indexOf(submits[i]) == -1) {
							fillImage(subm,"incorrect"); //false
						} else {
							fillImage(subm,"correct"); //ok
						}
						show(subm + "_img_label","inline");  // show mark on option
					}
				}
				var imageTds = getElementsByName("td", "MAansAnsw_" + qid);
					for (var i=0; i<imageTds.length; i++) {
						if (isIE){
							imageTds[i].style.display = "block"; 
						}else{
							imageTds[i].style.display = "table-cell"; 
						}
					}
				show(qid + "_corr_anw");  // show 'Correct response is...'

				setDisabled(qid); // mark question options disabled
				// remove events				
				jQuery("#WebMatchingActivity_" + qid).find("td").unbind();
				
				if (doneBtn != null) {
					submitBtn.style.visibility = "hidden";
					show(qid + "_doneBtn");
					doneBtn.disabled = false;
				}
			}
		}
		
		handleAdvice(qid, showAdvice, at_limit, qmatch); // show Advice
		handleHint(qid, currentQuestion, at_limit, qmatch); // show Hint
		handleFeedback(qid, sfeedback, at_limit, qmatch); // show Feedback
		
		setAttemptLimit(qid,at_limit);  // decrease attempts + update tailed text
		if (!redraw) {
			currentQuestion.setUserResult();//xxx send result to CRE here!
			setUserResults(); //xxx commented out for offline test only
		}
    }
} // end of some option was selected



// -----------------------------
// --- check submited values ---
// qid = question id, msg = message to display if no option is selected
function checkMASubmit(qid, msg, title) {
    var test = true;
    var user_answer = "";
    var re = /^.*_match_/; //to get simpl id from match id
    // take select elements for the form 
    var elms = getElementsByName("input", qid+"_select");
    
    for (var i=0; i < elms.length; i++) {
		if (elms[i].value == "") {
			test = false;
		}
		user_answer += elms[i].id.replace(re,"") + ":" + elms[i].value + "|";
    }
    if (!test) {
		if(msg) {
			jAlert(msg, title);
		}
		return test;
    } else {
		getQuestion(qid).setParam("userAnswer", user_answer);
		return user_answer;
    }
}

// -------------------------------
// --- init the whole CRE page ---

function initMAPage() {   
    setMAQuestions();
    getUserResults(maquestions);
    for (var i in maquestions) {
    	randomMAQ(maquestions[i].guid);		
    }
}
	
/**
* Randomize MAQ options (TD html tags) 
*/	
function randomMAQ(qid){
    if(random){
		var quest 	  = getQuestion(qid),
			MA        = jQuery("#WebMatchingActivity_"+qid+" div.MA").attr('id'),
			table     = jQuery("#"+MA+" table.MaTable"),
			questions = [],
			answers   = [];

		//return if questions haven't been initialized
		if (table == null || quest == null) {
			return;
		}

		//always randomize choices for WMA despite "randomize" parameter(only once)

		// on first init
		if (getArrangement(quest.guid) == null){

			// shuffle all options
			jQuery("#" + MA + " table.MaTable td[id^='MAque']").shuffle();
			jQuery("#" + MA + " table.MaTable td[id^='MAans']").shuffle();

			// save arrangement
			jQuery("#" + MA + " table.MaTable td > div.Distractor").each(function(i, selected){ 
				questions[i] = jQuery(selected).attr("id"); 
			});
			jQuery("#" + MA + " table.MaTable td > div.MatchSelect > input").each(function(i, selected){ 
				answers[i] = jQuery(selected).attr("id"); 
			});

			setArrangement(quest.guid, [questions, answers]);
		
		// if page was already initialized
		} else {
			var res   = getArrangement(quest.guid),
				a     = [],
				v     = [],
				q     = [],
				qHTML = [],
				aHTML = [],
				vHTML = [];

			questions = res[0];
			answers   = res[1];
		
			// restore saved arrangement

			qHTML = jQuery("#" + MA + " table.MaTable td[id^='MAque']");
			vHTML = jQuery("#" + MA + " table.MaTable td[name^='MAansAnsw']");
			aHTML = jQuery("#" + MA + " table.MaTable td[id^='MAans']");
		
			jQuery("#" + MA + " table.MaTable tr").each(function(i, sel){
				
				for (var ii = 0; ii < questions.length; ii++){

					if (questions[i] == jQuery(qHTML[ii]).find("div:first").attr("id"))
						q.push(jQuery(qHTML[ii]).html());

					if (answers[i] == jQuery(aHTML[ii]).find("input:first").attr("id")){
						a.push(jQuery(aHTML[ii]).html());
						v.push(jQuery(vHTML[ii]).html());
					}
				}
			});
			jQuery("#" + MA + " table.MaTable td[id*='MAque']").each(function(i, sel){jQuery(sel).html(q[i]);});
			jQuery("#" + MA + " table.MaTable td[name*='MAansAnsw']").each(function(i, sel){jQuery(sel).html(v[i]);});
			jQuery("#" + MA + " table.MaTable td[id*='MAans']").each(function(i, sel){jQuery(sel).html(a[i]);});
		}		

		/* correct background-color after shuffle*/
		jQuery("#" + MA + " table.MaTable td[class*='MaDistractor']:even").each(function(){
			var distractorTD = jQuery(this),
				matchTD      = jQuery(this).siblings("td[class^='MaMatch']");

			if(distractorTD.hasClass('MaDistractor'))
				distractorTD.removeClass('MaDistractor').addClass('MaDistractorCellDark')
			if(matchTD.hasClass('MaMatch')) 
				matchTD.removeClass('MaMatch').addClass('MaMatchCellDark');
		});
		
		jQuery("#" + MA + " table.MaTable td[class*='MaDistractor']:odd").each(function(){
			var distractorTD = jQuery(this),
				matchTD      = jQuery(this).siblings("td[class^='MaMatch']");

			if(distractorTD.hasClass('MaDistractorCellDark')) distractorTD.removeClass('MaDistractorCellDark').addClass('MaDistractor')

			if(matchTD.hasClass('MaMatchCellDark')) matchTD.removeClass('MaMatchCellDark').addClass('MaMatch')
		});
		/* end correct background-color*/
		
		jQuery("#" + MA + " table.MaTable td[name^='MAansAnsw']").each(function(){
				var answTD  = jQuery(this).siblings("td[id*='MAans']"),		
					matchID = jQuery(answTD).find("input").attr('id');

				jQuery(this).find("img").attr('id',matchID+'_img_label');
			});

		random = false;
	}
}

jQuery.fn.extend({
    disableTextSelection : function() {
            this.each(function() {
                    this.onselectstart = function() { return false; };
                    this.unselectable = "on";
                    jQuery(this).css('-moz-user-select', 'none');
            });
    }
});

jQuery(window).load(function(){
	//put tabindex to all cells and submit buttons
    var	tabIndex = 1; 
	var MAelements = jQuery('div.WebMatchingActivity');
	MAelements.each(function(){
		jQuery(this).disableTextSelection();
		var indexArea = jQuery(this).find('a')
			indexArea.each(function(){
				jQuery(this).attr('tabindex', tabIndex);
				tabIndex++;	
			})
		jQuery(this).find("input:button").attr('tabindex', tabIndex);
		tabIndex++;	
	})	
});

/**
* Find parent TD tag of element
*/
function findTD(elem){		
	var TD = elem;
		while(TD.tagName != 'TD' ) TD = TD.parentNode;  //find TD
	return TD
}