/*

Copyright c 2013 Xyleme, Inc., 1881 9th Street, Suite 300, Boulder, Colorado 80302 USA.
All rights reserved.

This file and related documentation are protected by copyright and
are distributed under licenses regarding their use, copying, distribution,
and decompilation. No part of this product or related documentation may
be reproduced or transmitted in any form or by any means, electronic or
mechanical, for any purpose, without the express written permission of
Xyleme, Inc.

*/

 
// --------------------------------------------------
// JS functions for WebMultipleChoice question
// --------------------------------------------------
//
// MCQuestion() : definition of Object type;
//   getParamValue() : get value of specified question parameter;
//   setParam() : set value for specified question parameter;
//   redraw() : redraw the question according the current data stored in question object; 
//   setUserResult() : store user answers;
// addMCQuestion() : add question to question array;
// getQuestion() : get question from question array;
// setMCQuestions() : set parameters for all questions in array;
//
// handleWebMCActivity() : evaluate user answer and invoke proper action;
// checkMCSubmit() : check if submit is valid;
// initMCPage() : init the CRE object;

// --------- following functions moved to questions.js ----------
// show() : show element of question form (image, feedback...);
// hide() : hide element of question form;
// setDisabled() : disable whole question;
// setAttemptLimit() : set number of attempts remaining;
// getHint() : get question hint if available;
// getUserResults() : update question object with any existing data present in CRE;
// setUserResults() : set user results to CRE;
//


scorableContent = true;   //Used by CRE to determine if the page is scorable
MCQ = true;               // defined in questions.js
var mcquestions = new Array();

// ---------------------------------
// --- definition of Object type ---
function MCQuestion() {
    this.randomize = false;
    this.redraw = function () {
            //redraw the attempts left (as it goes from XSLT so the number is incorrect)
            var at = document.getElementById(this.guid + "_at_limit"); // change text node informing of remaining attempts
		
			if (this.attempts == 1) {
				at.firstChild.nodeValue = "(" + this.attempts + " " + document.langAttemptRemaining + ")";
			} else  {
				at.firstChild.nodeValue = "(" + this.attempts + " " + document.langAttemptsRemaining + ")";
			}
			
            //set option
            // take radio input element or all checkbox elements for the form 
            if (jQuery(document.forms["QuestionForm_"+this.guid]).find("input[type='radio']").length > 0) {
                var elms = getElementsByName("input", "radio_"+this.guid);
                var type = 'radio';
            } else {
                var elms = document.forms["QuestionForm_"+this.guid].elements;
                var type = 'chbox';
            }
            
            // set checked options
          if (type != "chbox" || this.attempts == 0 || this.result == "correct") {
                for (var i=0; i<elms.length; i++) {
                  if(elms[i].value){
                    var answ = elms[i].value.split(this.guid +"_option_").join("");
                    if (this.userAnswer.indexOf(answ) == -1 ) {
                      continue;
                    }
                    elms[i].checked = true;
                  }
                }
            }
            //evaluate question
            //if question is done, add +1 attempt and call handleWebMCActivity 
            //  to render the question result. Attempts are decreased in this function by 1   
            //if question is not done yet (there are some spare attempts)
            //  show the clear checkboxes, right attempts 
            //  ommitted (AND all hints which were shown before)
            if (this.attempts == 0 || this.result == "correct") {
                this.attempts = this.attempts + 1; //increase attempts as handleWebMCActivity 
                                                   //will decrease it back when evaluating
                handleWebMCActivity(this.guid,"","",true);
            }            
        }
}

// Extend basic Question type
MCQuestion.prototype = new Question;

// --------------------------------------
// --- add question to question array ---
function addMCQuestion(question) {
    mcquestions[mcquestions.length] = question;
    addQuestion(question);
}

// TODO move "returnAttr" function to question.js when all questions WAB attributes will be updated with "data-" prefix
// this function also present in WebDndSequenceActivity.js
var MCQUtils = { 
	returnAttr : function(selected, obj){
					var attributePrefix = 'data-';
					if (obj.indexOf(attributePrefix) != -1){
						attributePrefix = '';
					}
					var attr = jQuery(selected).attr(attributePrefix + obj);
					return (typeof attr=='undefined' || attr=='undefined') ? null : attr;
				}
}

// -------------------------------------------------
// --- set parameters for all questions in array ---
function setMCQuestions() {
    jQuery(".WebMultipleChoice form[id*='WebMultipleChoice'] input.WebActivityBehavior").each(function(i, selected){
	
		var qid = jQuery(selected).attr('id').replace('wab_','');
		
        var newMCQuestion = new MCQuestion();
        newMCQuestion.setParam("guid", qid);
        newMCQuestion.setParam("randomize", MCQUtils.returnAttr(selected, "wab_RandomizeChildren"));
        newMCQuestion.setParam("attempts", MCQUtils.returnAttr(selected, "wab_AttemptLimit"));
        newMCQuestion.setParam("showHint", MCQUtils.returnAttr(selected, "wab_ShowHint"));
        newMCQuestion.setParam("showFeedback", MCQUtils.returnAttr(selected, "wab_ShowFeedback"));
        newMCQuestion.setParam("showAdvice", MCQUtils.returnAttr(selected, "wab_showAdvice"));
        newMCQuestion.setParam("hintActive", MCQUtils.returnAttr(selected, "wab_hint_active"));
        newMCQuestion.setParam("corectAnswer", Base64.decode(MCQUtils.returnAttr(selected, "wab_correct_answer")));        
        newMCQuestion.setParam("rollupWeight", MCQUtils.returnAttr(selected, "wab_RollupWeight"));
        newMCQuestion.setParam("leaveCorrectBetweenAttempts", MCQUtils.returnAttr(selected, "wab_LeaveCorrectBetweenAttempts"));

        addMCQuestion(newMCQuestion);
        randomMCQ(qid)
    });
}

/*   IE6 workaround  */
if(isIE6) {
	var it = 1;
	while(document.getElementById(it + "_Col2")){
		document.getElementById(it + "_Col2").style.marginLeft = "77px";
		it++;
	}
	var l_elements = document.getElementsByClassName('Col2Light');
	var d_elements = document.getElementsByClassName('Col2Dark');
	
	for(var j=0; j<l_elements.length; j++) {
		l_elements[j].className = "Col2Light_IE6";
	}
	
	for(var j=0; j<d_elements.length; j++) {
		d_elements[j].className = "Col2Dark_IE6";
	}
}


// ------------------------------------------------------------
// --- handling submited form of WebMultipleChoice question ---
function handleWebMCActivity(qid, msg, title,redraw) {
	// ---imagemapquiz variables---
	var submitBtn = document.getElementById(qid + "_submitBtn");
	var doneBtn = document.getElementById(qid + "_doneBtn");
	// ---imagemapquiz variables---
	
    var submits = String(checkMCSubmit(qid, msg, title)); // id of selected option
    submits = submits.split("|");
    
    if (submits[0] != "false") { //some option WAS selected
        var currentQuestion = getQuestion(qid);
        var at_limit = currentQuestion.getParamValue("attempts"); // attempt_limit
        var anws = currentQuestion.getParamValue("corectAnswer"); // get correct answer(s)
        anws = anws.split("|"); // split them by "|" to the anws array
		
	var sfeedback = getFeedbackMode(currentQuestion);		
	var showAdvice = currentQuestion.getParamValue("showAdvice");
	  
        //separate code for radio-buttons and check-boxes
		var radio = getElementsByName("input", "radio_"+qid).length != 0 ? true : false ;
		var qmatch = true; // flag for all answers correct
		
        if (radio) { //radio

            var anw        = qid + "_option_" + anws[0];    // create correct   answer id from question id
                submits[0] = qid + "_option_" + submits[0]; // create submitted answer id from question id 


            if (submits[0] == anw) { ////// correct answer
                qmatch = true;
				currentQuestion.setParam("result", "correct"); // set question result
                if (sfeedback === ShowFeedback.all) {
					fillImage(submits[0],"correct");
					show(submits[0] + "_img_label","inline");  // show image in front of option 
					show(submits[0] + "_feedback_cr"); // show correct label
					show(submits[0] + "_feedback", "", true); // show feedback, if available
				}
                setDisabled(qid); // mark question options disabled
				if (doneBtn != null) {
					submitBtn.style.visibility = "hidden";
					show(qid + "_doneBtn");
					doneBtn.disabled = false;
				}
				
            } else {  ////// incorrect answer
                qmatch = false;
				currentQuestion.setParam("result", "incorrect"); // set question result
                if (at_limit == 1) { //not last attempt
                    if (sfeedback === ShowFeedback.all) {
                        fillImage(submits[0],"incorrect");
                        show(submits[0] + "_img_label","inline");  // show image in front of option 
                        //show correct response
                        show(submits[0] + "_feedback_ir"); // show incorrect label
                        //Yum demo --
                        fillImage(anw,"also_correct");
                        show(anw + "_img_label","inline");  // show image in front of option 
                        show(anw + "_feedback_tr"); // show 'This is the...' text
                        show(anw + "_feedback", "", true); // show feedback, if available
                         //Yum demo end --
                        show(submits[0] + "_feedback", "", true); // show response feedback
                    }					
					setDisabled(qid); // mark question options disabled
					if (doneBtn != null) {
						submitBtn.style.visibility = "hidden";
						show(qid + "_doneBtn");
						doneBtn.disabled = false;
					}
                }
            }
        } else { //checkboxes
            // find out, if answer is correct
            // take all submits and check them 1 by 1 against correct answers

            // create submitted answers ids from question id
            for (var i in submits) {
                submits[i] = qid + "_option_" + submits[i];
            }

            for (var i=0; i < anws.length-1; i++) {
                var match = false;
                var anw = qid + "_option_" + anws[i]; // create answer id from question id
                for (var j=0; j < submits.length-1; j++) {
                    if (submits[j] == anw) {
						match = true; //submits[j] is correct answer
					}
                }
                if (!match) {
					qmatch = false; // flag for whole question - correct/incorrect
				}
            }
            for (var i=0; i < submits.length-1; i++) {
                var match = false;
                for (var j=0; j < anws.length-1; j++) {
                    var anw = qid + "_option_" + anws[j]; // create answer id from question id
                    if (submits[i] == anw) {
						match = true; //submits[i] is correct answer
					}
                }
                if (!match) {
					qmatch = false; // flag for whole question - correct/incorrect
				}
            }

            if (qmatch == true) { /////////// CORRECT ANSWER!
                currentQuestion.setParam("result", "correct"); // set question result
                if (sfeedback === ShowFeedback.all) {
					for (var i=0; i < submits.length-1; i++) {
						fillImage(submits[i],"correct");
						show(submits[i] + "_img_label","inline");  // show image in front of option 
						show(submits[i] + "_feedback_cr"); // show correct label
						show(submits[i] + "_feedback", "", true); // show feedback, if available
					}
				}
                setDisabled(qid); // mark question options disabled
				if (doneBtn != null) {
					submitBtn.style.visibility = "hidden";
					show(qid + "_doneBtn");
					doneBtn.disabled = false;
				}	
            } else { ///////// INCORRECT ANSWER! (not all correct anws checked or one or more incorrect anws)
                
                //if last attempt show also correct answers (depending on feedback) and disable question

                currentQuestion.setParam("result", "incorrect"); // set question result

                if (at_limit > 1) {
					var leaveAnw = currentQuestion.getParamValue("leaveCorrectBetweenAttempts");
					
                    for (var i=0; i < submits.length-1; i++) {
						document.getElementById("chbox_"+submits[i]).checked = false;
					}
					
					if (leaveAnw == "true") {
						for (var i=0; i < submits.length-1; i++) { //mark correct/incorrect submitted answers
							var iscorr = false
							for (var j=0; j < anws.length-1; j++) {
								var anw = qid + "_option_" + anws[j]; // create answer id from question id
								if (submits[i] == anw) {
									iscorr = true;
									document.getElementById("chbox_" + submits[i]).checked = true;
								}
							}
											
							if (iscorr && sfeedback == "true") {
								fillImage(submits[i],"correct");
								show(submits[i] + "_img_label","inline");  // show image in front of option 
							}
						}
					}
                }
                else {
                    if (sfeedback === ShowFeedback.all) {
                        for (var i=0; i < submits.length-1; i++) { //mark correct/incorrect submitted answers
                            var iscorr = false
                            for (var j=0; j < anws.length-1; j++) {
                                var anw = qid + "_option_" + anws[j]; // create answer id from question id
                                if (submits[i] == anw) iscorr = true;
                            }
                        if (iscorr) {
                                fillImage(submits[i],"correct");
                                show(submits[i] + "_img_label","inline");  // show image in front of option 
                                show(submits[i] + "_feedback_cr"); // show incorrect label
                            } else  {
                                fillImage(submits[i],"incorrect");
                                show(submits[i] + "_img_label","inline");  // show image in front of option 
                                show(submits[i] + "_feedback_ir"); // show incorrect label
                            }
                            show(submits[i] + "_feedback", "", true); // show response feedback
                        }
                        for (var j=0; j < anws.length-1; j++) { // mark 'also correct' answers
                            var isalsocorr = true;
                            var anw = qid + "_option_" + anws[j]; // create answer id from question id
                            for (var i=0; i < submits.length-1; i++) {
                                if (submits[i] == anw) isalsocorr = false
                            }
                            if (isalsocorr) {
                                fillImage(anw,"also_correct");
                                show(anw + "_img_label","inline");  // show image in front of option 
                                show(anw + "_feedback_tr"); // show 'This is the...' text
                                show(anw + "_feedback", "", true); // show response feedback
                            }
                        }
                    }
                    setDisabled(qid); // mark question options disabled
					if (doneBtn != null) {
						submitBtn.style.visibility = "hidden";
						show(qid + "_doneBtn");
						doneBtn.disabled = false;
					}
                }
            }
        }
		
		handleAdvice(qid, showAdvice, at_limit, qmatch); // show Advice 
		handleHint(qid, currentQuestion, at_limit, qmatch); // show Hint
		handleFeedback(qid, sfeedback, at_limit, qmatch); // show Feedback
		
        setAttemptLimit(qid,at_limit);  // decrease attempts + update tailed text
        if (!redraw) {
            currentQuestion.setUserResult();//send result to CRE here!
            setUserResults();
        }
    } // end of some option was selected
}
        
// -----------------------------
// --- check submited values ---
// qid = question id, msg = message to display if no option is selected
function checkMCSubmit(qid, msg, title) {
    var test = false;
    var user_answer = "";

	var elms = getElementsByName("input","radio_"+qid).length != 0 ? 
							getElementsByName("input","radio_"+qid) : 
							getElementsByName("input","chbox_"+qid) ;
	
    for (var i=0; i < elms.length; i++) {
        if (elms[i].checked != "") {
            test = true;
            // save only N****
            var s = elms[i].value;
            user_answer += s.substring(s.lastIndexOf('_')+1, s.length) + "|";
        }
    }
    if (!test) {
        if(msg) jAlert(msg, title);
        return test;
    } else {
        getQuestion(qid).setParam("userAnswer", user_answer);
        return user_answer;
    }
}

// -------------------------------
// --- init the whole CRE page ---

function initMCPage() {  

    setMCQuestions();
    getUserResults(mcquestions);
}

function randomMCQ(qid){
	var quest = getQuestion(qid);
	if (quest==null)
		return;
	var rand = quest.getParamValue("randomize");
	if (rand != "true")
		return;
	var	$MCQ_options = jQuery('#WebMultipleChoice_' + qid).find("div[class='Option']");
	if ($MCQ_options == null || $MCQ_options.length < 2)
		return;
	var question = qHTML = [];
	if (getArrangement(quest.guid) == null){
		$MCQ_options.shuffle();
	}
	qHTML=$MCQ_options;

	if (getArrangement(quest.guid) == null){
		$MCQ_options.each(function(i, selected){ 
			question[i] = jQuery(selected).attr("id"); 
		});
		setArrangement(quest.guid, [question, [""]]);
	} else {
		var rez = getArrangement(quest.guid);
		question = rez[0];
		var q = new Array();
		$MCQ_options.each(function(i, sel){
			for (var ii=0; ii < question.length; ii++){
				if (question[i] == jQuery(qHTML[ii]).attr("id"))
					q.push(jQuery(qHTML[ii]).html());
			}
		});
		$MCQ_options.each(function(i, sel){jQuery(sel).html(q[i]);});
	}
	/* correct background-color after shuffle*/
	jQuery("#WebMultipleChoice_"+qid+" div.Option:even").each(function(){	       
			var col1 = jQuery(this).children('div:first')
			var col2 = jQuery(this).find('div.Col2').children('div:first')
			if(col1.hasClass('Light')) col1.removeClass('Light').addClass('Dark')
			if(col2.hasClass('Col2Light')) col2.removeClass('Col2Light').addClass('Col2Dark')			
	})
	
	jQuery("#WebMultipleChoice_"+qid+" div.Option:odd").each(function(){	       
			var col1 = jQuery(this).children('div:first')
			var col2 = jQuery(this).find('div.Col2').children('div:first')
			if(col1.hasClass('Dark')) col1.removeClass('Dark').addClass('Light')
			if(col2.hasClass('Col2Dark')) col2.removeClass('Col2Dark').addClass('Col2Light')			
	})	
	/* end correct background-color*/
}