/*

Copyright c 2013 Xyleme, Inc., 1881 9th Street, Suite 300, Boulder, Colorado 80302 USA.
All rights reserved.

This file and related documentation are protected by copyright and
are distributed under licenses regarding their use, copying, distribution,
and decompilation. No part of this product or related documentation may
be reproduced or transmitted in any form or by any means, electronic or
mechanical, for any purpose, without the express written permission of
Xyleme, Inc.

*/

 
// --------------------------------------------------
// JS functions for WebSequenceActivity question
// --------------------------------------------------
//
// Question() : definition of Object type;
//   getParamValue() : get value of specified question parameter;
//   setParam() : set value for specified question parameter;
//   redraw() : redraw the question according the current data stored in question object; 
//   setUserResult() : store user answers;
// addSAQuestion() : add question to question array;
// getQuestion() : get question from question array;
// setSAQuestions() : set parameters for all questions in array;
//
// handleWebSequenceActivity() : evaluate user answer and invoke proper action;
// checkSASubmit() : check if submit is valid;
// initSAPage() : init the CRE object;

// ------------ moved to questions.js ---------------
// show() : show element of question form (image, feedback...);
// hide() : hide element of question form;
// setDisabled() : disable whole question;
// setAttemptLimit() : set number of attempts remaining;
// getHint() : get question hint if available;
// getUserResults() : update question object with any existing data present in CRE;
// setUserResults() : set user results to CRE;
//

// lines marked with xxx are commented/uncommented depending on onCRE/offCRE environment


scorableContent = true;   //Used by CRE to determine if the page is scorable
SAQ = true;               // defined in questions.js
var saquestions = new Array();

// ---------------------------------
// --- definition of Object type ---
function SAQuestion()
{
    this.randomize = false;
    this.redraw = function ()
	{
	    //redraw the attempts left (as it goes from XSLT so the number is incorrect)
	    var at = document.getElementById(this.guid + "_at_limit"); // change text node informing of remaining attempts
			if (this.attempts == 1) at.firstChild.nodeValue = "(" + this.attempts + " " + document.langAttemptRemaining + ")";
			else  at.firstChild.nodeValue = "(" + this.attempts + " " + document.langAttemptsRemaining + ")";

	    //set option
	    // take all select elements for the form 
	    var user_anws = String(this.userAnswer).split("|");
	    
	    // set checked options
	    if (this.attempts == 0 || this.result == "correct") {
		for (var i=0; i<user_anws.length-1; i++) {
		    var anw = user_anws[i].split(":");
		    //alert(this.guid + '_option_' + anw[0]);	
		    var oEl = document.getElementById(this.guid + '_option_' + anw[0]);
			
		    if (oEl != null) {
			    oEl.value = anw[1];
		    }
		}
	    }

	    //evaluate question
	    if (this.attempts == 0 || this.result == "correct") {
		this.attempts = this.attempts + 1; //increase attempts as handleWebSequenceActivity 
		                                   //will decrease it back when evaluating
		handleWebSequenceActivity(this.guid,"", "", true);
	    }
	    
// 	    if (this.result == "correct") {
// 		// show image in front of option 
// 		for (var i=0; i<user_anws.length-1; i++) {
// 		    var anw = user_anws[i].split(":");
// 		    markSAOption(this.guid,this.guid+'_option_'+anw[0],"correct");
// 		}
// 		setDisabled(this.guid); // mark question options disabled
		
// 	    } else {
// 		if (this.result == "incorrect") {
// 		    if (this.attempts == 0) {
// 			for (var i=0; i<user_anws.length-1; i++) {
// 			    var anw = user_anws[i].split(":");
// 			    markSAOption(this.guid,this.guid+'_option_'+anw[0],"incorrect");
// 			}
// 		    }
// 		}
// 		if (mode == "review" || this.attempts == 0) {    // in case of review - show correct answer
// 		    var anws = String(this.corectAnswer).split("|"); // mark also correct options
// 		    for (var i=0; i<anws.length-1; i++) {
// 		    anw = anws[i].split(':');
// 		    markSAOption(this.guid,this.guid+"_option_"+anw[0],"also_correct"); 
// 		    }
// 		    setDisabled(this.guid);
// 		}
// 		if (!this.attempts > 0) {
// 		    setDisabled(this.guid);
// 		}
// 	    }
	}
}

// Extend basic Question type
SAQuestion.prototype = new Question;

// --------------------------------------
// --- add question to question array ---
function addSAQuestion(question)
{
    saquestions[saquestions.length] = question;
    addQuestion(question);
}

// -------------------------------------------------
// --- set parameters for all questions in array ---
function setSAQuestions()
{
    for (l=0; l<document.forms.length; l++) {
        var fname = document.forms[l].name;     
        if (fname.toLowerCase().indexOf("questionform") == -1) continue;
        var fid = document.forms[l].id;
        if (fid.toLowerCase().indexOf("websequenceactivity") == -1) continue;

        var wab_id = fname.substr(13);

        var wab_obj = document.getElementById("wab_"+wab_id);
        newSAQuestion = new SAQuestion();
        newSAQuestion.setParam("guid", wab_id);
        var params = wab_obj.attributes;    
        var param;

	newSAQuestion.setParam("randomize", 
		wab_obj.getAttribute("wab_RandomizeChildren"));	
	newSAQuestion.setParam("questionRemediation", 
		wab_obj.getAttribute("wab_questionRemediation"));	
	newSAQuestion.setParam("attempts", 
		wab_obj.getAttribute("wab_AttemptLimit"));	
	newSAQuestion.setParam("showHint", 
		wab_obj.getAttribute("wab_ShowHint"));	
	newSAQuestion.setParam("showFeedback", 
		wab_obj.getAttribute("wab_ShowFeedback"));
    newSAQuestion.setParam("showAdvice", 
        wab_obj.getAttribute("wab_showAdvice"));		
	newSAQuestion.setParam("hintActive", 
		wab_obj.getAttribute("wab_hint_active"));	
	newSAQuestion.setParam("corectAnswer", 
		Base64.decode(wab_obj.getAttribute("wab_correct_answer")));
    newSAQuestion.setParam("rollupWeight", 
        wab_obj.getAttribute("wab_RollupWeight"));
	newSAQuestion.setParam("leaveCorrectBetweenAttempts", 
        	wab_obj.getAttribute("wab_LeaveCorrectBetweenAttempts"));

        addSAQuestion(newSAQuestion);
    }
}

// --------------------------------------------------------------
// --- handling submited form of WebSequenceActivity question ---
function handleWebSequenceActivity(qid, msg, title, redraw)
{
	// ---imagemapquiz variables---
	var submitBtn = document.getElementById(qid + "_submitBtn");
	var doneBtn = document.getElementById(qid + "_doneBtn");
	// ---imagemapquiz variables---
    //justforTest(); //xxx init saquestions for offCRE testing 
    
    var submits = String(checkSASubmit(qid, msg, title)); // id of selected option
    submits = submits.split("|");
    
    if (submits[0] != "false") { //some option WAS selected
		
	var currentQuestion = getQuestion(qid);
	var at_limit = currentQuestion.getParamValue("attempts"); // attempt_limit
	var answer = currentQuestion.getParamValue("corectAnswer"); // get correct answer(s)
	
	var anws = answer.split("|"); // split them by "|" to the anws array
	
	var sfeedback = getFeedbackMode(currentQuestion);
	
	var showAdvice = currentQuestion.getParamValue("showAdvice");
	
	// find out, if answer is correct
	// take all submits and check them 1 by 1 against correct answers
	var qmatch = true; // flag for all answers correct
	for (var i=0; i < submits.length-1; i++) {
	    var subm = submits[i].split(":"); // subm[0] is select id, subm[1] is select value
	    if (subm[0] != anws[subm[1]-1]) qmatch = false; //submits[i] is incorrect answer
	}	

	if (qmatch == true) { /////////// CORRECT ANSWER!
	    currentQuestion.setParam("result", "correct"); // set question result
	    for (var i=0; i < submits.length-1; i++) {
		var subm = submits[i].split(":");
		markSAOption(qid,qid+"_option_"+subm[0],'correct'); // mark options as correct
	    }
	    setDisabled(qid); // mark question options disabled
		if (doneBtn != null) {
			submitBtn.style.visibility = "hidden";
			show(qid + "_doneBtn");
			doneBtn.disabled = false;
		}
		
	} else { ///////// INCORRECT ANSWER! (one or more incorrect options)
	    currentQuestion.setParam("result", "incorrect"); // set question result	    
		if (at_limit > 1) {
			var leaveAnw = currentQuestion.getParamValue("leaveCorrectBetweenAttempts");
			if (leaveAnw == "true") {
				for (var i=0; i < submits.length-1; i++) {
					var subm = submits[i].split(":"); // subm[0] is select id, subm[1] is select value
				
					if (subm[0] == anws[subm[1]-1]) {
						markSAOption(qid,qid+"_option_"+subm[0],"correct");
					} else {
						document.getElementById(qid + "_option_" +subm[0]).value="";
						markSAOption(qid,qid+"_option_"+subm[0],"transparent");
					}
				
				
				}	
			} else {
				for (var i=0; i < submits.length-1; i++) {
					var subm = submits[i].split(":"); 
					document.getElementById(qid + "_option_" +subm[0]).value="";
				}
			}
			
		markSAOption(qid,qid,'question'); // mark question as incorrect		
		//clean input fields
		//var elms = document.getElementsByName(qid + "_select");
		//for (var i=0; i < elms.length; i++) {
		//    elms[i].value = "";
		//}		
	    }
	    else {
		if (sfeedback === ShowFeedback.all) {
		    for (var i=0; i < submits.length-1; i++) {
			var subm = submits[i].split(":");
			if(subm[0] == anws[subm[1]-1]) markSAOption(qid,qid+"_option_"+subm[0],'correct');
			else markSAOption(qid,qid+"_option_"+subm[0],'incorrect');
		    }
		    markSAOption(qid,qid,'also_correct');

		} else {
		    markSAOption(qid,qid,'question'); // mark question as incorrect
		}		
      		setDisabled(qid); // mark question options disabled
			if (doneBtn != null) {
				submitBtn.style.visibility = "hidden";
				show(qid + "_doneBtn");
				doneBtn.disabled = false;
			}
	    }
	}
	
	handleAdvice(qid, showAdvice, at_limit, qmatch); // show Advice
	handleHint(qid, currentQuestion, at_limit, qmatch); // show Hint
	handleFeedback(qid, sfeedback, at_limit, qmatch); // show Feedback
	
	setAttemptLimit(qid,at_limit);  // decrease attempts + update tailed text
	if (!redraw) {
	    currentQuestion.setUserResult();//send result to CRE here!
	    setUserResults(); //xxx for offCRE testing
	}
    } // end of some option was selected
}


// ----------------------------------------------------------------
// --- handles complex marking of options as correct, incorrect ---
function markSAOption(qid, option_id, how)
{
    var currentQuestion = getQuestion(qid);
    var sfeedback = currentQuestion.getParamValue("showFeedback");
	if (sfeedback != "true") {
		return;
	}
    var re = /_option.*$/gi;
    var qid = option_id.replace(re,""); // question id
    var currentQuestion = getQuestion(qid);
    var shint = currentQuestion.getParamValue("showHint");
    var at_limit = currentQuestion.getParamValue("attempts"); // attempt_limit
	
    // show correct answer label
    if (how == "correct") {
	fillImage(option_id,"correct");
	show(option_id + "_img_label","inline");  // show image in front of option 
    } 
    //show incorrect answer label
    else if (how == "incorrect") {
	fillImage(option_id,"incorrect");
	show(option_id + "_img_label","inline");  // show image in front of option 
    }
    // show also label of correct options
    else if (how == "also_correct") {
	show(qid + "_corr_anw", "", true);  // show 'Correct answer is...'
    }
    else if (how == "transparent") {
	fillImage(option_id,"transparent");
	show(option_id + "_img_label","inline");  // show image in front of option 
    } 
}


// -----------------------------
// --- check submited values ---
// qid = question id, msg = message to display if no option is selected
function checkSASubmit(qid, msg, title)
{
    var test = true;
    var user_answer = "";
    var re = /^.*_option_/; //to get simpl id from option id
    // take select elements for the form 
    var elms = document.getElementsByName(qid+"_select");
    for (var i=0; i < elms.length; i++) {
	if (elms[i].value == "") test = false;
	user_answer += elms[i].id.replace(re,"") + ":" + elms[i].value + "|";
    }
    if (!test) {
	if(msg) jAlert(msg, title);
	return test;
    } else {
	//self.alert("user_answer: "+user_answer); //debug
	getQuestion(qid).setParam("userAnswer", user_answer);
	return user_answer;
    }
}

// -------------------------------
// --- init the whole CRE page ---

function initSAPage()
{   
    //    alert("ga-initSAPage()");
    setSAQuestions();
    getUserResults(saquestions);
}

//window.onload = initQuestionPage(); //xxx for offCRE testing

//function justforTest() //xxx for offCRE testing
//{ 
//    setSAQuestions(); //xxx
//    getUserResults(); //xxx
//}
