/*

Copyright c 2013 Xyleme, Inc., 1881 9th Street, Suite 300, Boulder, Colorado 80302 USA.
All rights reserved.

This file and related documentation are protected by copyright and
are distributed under licenses regarding their use, copying, distribution,
and decompilation. No part of this product or related documentation may
be reproduced or transmitted in any form or by any means, electronic or
mechanical, for any purpose, without the express written permission of
Xyleme, Inc.

*/

(function($){
	// global variables
	var animateRes = true, // change to false to disable animation
		interval = 700,
		globalDelay = interval;
		
	function getCRE(win) {
		// global
		CRE = null;
		CRE_frame = null;

		// local
		var	maxTries = 50,
			nFindAPITries = 0;

		function ScanForCRE(win) {
			while ((win.CRE == null) && (win.parent != null) && (win.parent != win)) {
				nFindAPITries++;
					if (nFindAPITries > maxTries) {
						alert("Error in finding Europa-CRE instance -- too deeply nested.");
						break;
					}
					win = win.parent;
				}
				CRE_frame = win;
				return win.CRE;
		}

		nFindAPITries = 0;
		if ((win.parent != null) && (win.parent != win)) {
			CRE = ScanForCRE(win.parent);
		} 
		if ((CRE == null) && (win.opener != null)) {
			CRE = ScanForCRE(win.opener);
		}
	}
	
	function showTitle() {
		var $title = $('.container-title'),
			title  = CRE.TREE.getParentContainerTitle(),
			str = CRE.TREE.isRootLevelActivity() ? document.langCourseName || "Course Name :" : document.langSectionName || "Section Name :";

		$title.children('.caption').html(str);
		$title.children('.value').html(title);
	}
	
	function showScore(){
		var $score = $('.container-score'),
			container_score = Math.round( CRE.TREE.getParentTotalScore() ),
			passing_score = Math.round( CRE.TREE.getParentPassingScore() );

		$score.children('.caption').html(document.langCurrentScorePassingScore || "Current Score / Passing Score :");	
		$score.children('.value').html(container_score + '% / ' + passing_score + '%');
	}

	function showSectionWeight() {
		var $weight = $('.container-weight'),
			weight = Math.round( CRE.TREE.getContainerWeight() );

		// weight == 0 if author set 0 or 1 or weight wasn't authored - so hide container
		if (weight === 0) {
			$weight.hide();
			return;	
		}

		$weight.children('.caption').html(document.langSectionWeight || "Section Weight :")
		$weight.children('.value').html(weight + '%');
	}

	function showCustomFeedback(index) {
			var	score = CRE.TREE.getContainerTotalScore(index),
			intervals = [];

		$('#score-range-messages-' + index + ' > .score-range-message').each(function(i) {
			var $this = $(this);
			$this.attr('data-min', $this.attr('data-min') || 0).attr('data-max', +$this.attr('data-max') || 100);
			intervals.push( [ $this.attr('data-min'), $this.attr('data-max') ] );
		});

		//find score in intervals and show if found
		for(var i = 0, n = intervals.length; i < n; i++) {
			if (score >= intervals[i][0] && score <= intervals[i][1]) {				
				var message = $('#score-range-messages-' + index + ' .score-range-message[data-min=' + intervals[i][0] + '][data-max=' +  intervals[i][1] + ']').html();
				// if index equals -1 - main feedback, otherwise - section feedback 
				if(index === -1 && $.trim(message) !== '') {
						// add main feedback
						var $customFeedback = $('#custom-feedback-' + index);
						$customFeedback.html(message);
						if(animateRes) {
							$customFeedback.addClass('show-custom-feedback');
						} else {
							$customFeedback.removeClass('hidden');
						}
				}
				break;
			}
		}
		// return section feedback
		return message;
	}

	function handleRetake(){
		var $retake = $('.retake'),
			$input = $('.retake input'),
			container_score = CRE.TREE.getParentTotalScore(),
			passing_score = CRE.TREE.getParentPassingScore(),
			container_attempts = CRE.TREE.getContainerAttemptsCount(),
			containers_count = CRE.TREE.getScoredContainerPageCount(),
			retryLimit = $('#score-card-wrapper').data('retry_limit'),
			remaining_attempts = retryLimit + 1 - container_attempts;

		if (retryLimit === '' || retryLimit < 0) {
			retryLimit = Infinity;
		}

		if (container_score < 100 && retryLimit >= container_attempts) {
			if(animateRes){
				$retake.addClass('show-retake');
			} else {
				$retake.removeClass('hidden');
			}

			// show remaining retake attemps count
			// Here replace strings with strings from lang.js file
			if (remaining_attempts === 1) {
				remaining_attempts = '1 ' + document.langAttemptRemaining;
			} else {
				remaining_attempts = remaining_attempts + ' ' + document.langAttemptsRemaining;
			}
			if (retryLimit === Infinity) remaining_attempts = '';
			$retake.children('.remained-attempts').html(remaining_attempts)

			//customize button "value"
			$input.val(containers_count > 1 ? document.langRetakeAll || "Retake All" : document.langRetake || "Retake");

		} else {
			$retake.hide();
		}

		$input.click(function() {
			var msg = document.langRetakeConfirmMessage || "Are you sure you want to retake this assessment?";
			$('<div>' + msg + '</div>').dialog({
				title: 'Retake Confirmation',
				dialogClass: 'confirmation-dialog',
				modal: true,
				draggable: true,
				resizable: false,
				closeOnEscape : false,
				appendTo: 'parent',
				buttons : [ 
					{ 
						text: "Ok",
						tabIndex: -1, 
						click: function() { 
							CRE.TREE.retake('', true);					
						} 
					},
					{
						text: 'Cancel',
						tabIndex: -1,
						click: function() {
							$( this ).dialog( "close" );
						}
					}
				]
			});
		});
	}

	function showAssesments() {
		var scoredContainerPageCount = CRE.TREE.getScoredContainerPageCount(),
			$questions_container = $('.question-list'),
			ShowQuestionList = $('#score-card-wrapper').attr('data-show_question_list') != 'false',
			ReviewQuestions = $('#score-card-wrapper').attr('data-review_questions') != 'false',
			questions_table = '',			
			_even = false;

		// build table of questions
		if (scoredContainerPageCount == 0) { 
			questions_table = '<p class="message">You have not attempted any question in this course</p>'; //todo
		} else {

			for (var i=0; i<scoredContainerPageCount; i++) {

				var scoredPageCount = CRE.TREE.getScoredPageCount(i);
				if (scoredPageCount == 0) {
					questions_table += '<p class="message">You have not attempted any question in this container</p>';
				} else {
					questions_table += '<div class="questions-list-item">';

					// add headers to tables if agregated scorecard
					if (scoredContainerPageCount > 1) {

						questions_table += showAgregatedInfo(i);

						
					}

					questions_table += '<table border="0" cellpadding="0" cellspacing="0"><tbody>';
					for (var j=0; j<scoredPageCount; j++) {

						var questionCount = CRE.TREE.getQuestionCount(i, j);

						for (var k = 0; k < questionCount; k++) {
							// insert image
							questions_table += (_even) ? '<tr class="tr-even"><td class="q-image">' : '<tr><td class="q-image">';
							var result = CRE.TREE.getResultForQuestion(i, j, k);
							switch (result) {
								case 'correct'   : questions_table += '<span class="correct result"></span>'; break;
								case 'incorrect' : questions_table += '<span class="incorrect result"></span>'; break;
								default          : questions_table += '<span class="result">N/A</span>'; result = false;
							}

							// insert questions link
							questions_table += '</td><td class="q-link">';
							if(result && ReviewQuestions){
								questions_table += '<a href="javascript:reviewQuestionPage(' + i + ',' + j + ',\'' +  CRE.TREE.getPageAddress(i, j, k) + '\'); ">' 
								+ (qNamesMap[CRE.TREE.getQuestionGuid(i, j, k)] || CRE.TREE.getPageTitle(i, j, k)) + '</a>';
							} else {
								questions_table +=  qNamesMap[CRE.TREE.getQuestionGuid(i, j, k)] || CRE.TREE.getPageTitle(i, j, k);
							}

							// insert questions weight
							/*questions_table += '</td><td class="q-weight">';
							if (CRE.TREE.getWeightForQuestion) {
								var weight = (result == 'correct') ? CRE.TREE.getWeightForQuestion(i, j, k) : 0;
								questions_table += ( weight === null || isNaN(weight)) ? '' : weight*100 + '%'; 
							}*/
							questions_table += '</td></tr>';
							_even = !_even;
						}
					}
				}
				questions_table += '</tbody></table></div>';
			}
			if (ShowQuestionList) {
				$questions_container.html(questions_table);
			}
		}
	}

	function showAgregatedInfo(i) {
		var name_msg = document.langSectionName || "Section Name :",
			score_msg = document.langCurrentScorePassingScore || "Current Score / Passing Score :",
			weight_msg = document.langSectionWeight || "Section Weight :",
			ret = '';
		ret += '<div class="section">';

		ret += '<div class="section-title"><div class="caption">' + name_msg + '</div><div class="value">' + CRE.TREE.getContainerPageTitle(i) + '</div></div>';

		ret += '<div class="section-score"><div class="caption">' + score_msg +'</div><div class="value">' + CRE.TREE.getContainerTotalScore(i) + '% / ' + CRE.TREE.getPassingScoreForActivity(i) + '%</div></div>';

		var weight = CRE.TREE.getContainerWeight(i);
		if (weight !== 0) {
			ret += '<div class="section-weight"><div class="caption">' + weight_msg + '</div><div class="value">' + weight + '%</div></div>';	
		}
		var message = showCustomFeedback(i);
		if (message) {
			ret += '<div id="custom-feedback-' + i + '" class="section-feedback">' + message + '</div>';
		}
						
		ret += '</div>';
		return ret;
	}

	reviewQuestionPage = function(containerNum, pageNum, pageGUID) {
		W_CRE.containerNumber = containerNum;
		W_CRE.pageNumber = pageNum;	

		// show review dialog
		var path = pageGUID + '&mode=review';
		jQuery("#Scorecard-preview-content").attr("src", path);
		showDialog("#Scorecard-preview-content");
	}
	
	function animateResuts(){
		$element = $('.question-list');
		if(animateRes){
			// change css styles for animation					
			$element.css({width : "20%", opacity : 0.1}).removeClass('hidden');
			$element.find(".result").css({opacity: 0, top: "-30px"});
			$element.animate({ width: "100%", opacity: 1},
						{
							duration: interval,
							complete :  function(){
								var delay = 0;
								// animate each result
								$element.find(".result").each(function(){
									var $this = $(this);
									setTimeout(function(){
										$this.animate({top : 0, opacity: 1 }, 'slow', 'easeOutBounce');
									}, delay);
									delay += 100;
								});
							}
						})
			globalDelay += $element.find(".result").length * 100;
			// animate custom feedback
			$element.parent().find('.show-custom-feedback').each(function(){
				var $this = $(this);
				$this.css({opacity : 0, left: "-200px"}).removeClass('hidden');
				setTimeout(function(){
					$this.animate({opacity : 1, left: 0}, 'slow', 'easeInOutElastic');
				}, globalDelay);
				globalDelay += 600;
			});
			// animate retake button
			$element.parent().find('.show-retake').each(function(){
				var $this = $(this);
				$this.css("opacity" ,"0").removeClass('hidden');
				setTimeout(function(){
					$this.animate({opacity : 1}, 'slow');
				}, globalDelay);
			});				
		} else {
			$element.removeClass('hidden');
		}
	}

	function W_CRE_proto(){
		this.containerNumber;
		this.pageNumber;
		this.questionNumber;
		this.getAnswer = function(interactionId){
			return CRE.TREE.getInteractionPropertyValue(this.containerNumber, this.pageNumber, interactionId, 'learner_response');
		}
		this.getAttemptCount = function(guid){
			return 0;
		}			
		this.getResult = function(interactionId){            	
			return CRE.TREE.getInteractionPropertyValue(this.containerNumber, this.pageNumber, interactionId, 'result');
		}            
		this.advanceQuestions = function(){
			this.questionNumber++;
		}
	}
	W_CRE = new W_CRE_proto();

	$(function(){
		// initialization
		getCRE(window);

		showAssesments();
		showTitle();
		showScore();
		showSectionWeight();
		showCustomFeedback(-1); // add main custom feedback
		handleRetake();
		animateResuts();
		
		initializeDialog("#Scorecard-preview-content", "scorecardPreview");
	});

}(jQuery));