/*

Copyright c 2013 Xyleme, Inc., 1881 9th Street, Suite 300, Boulder, Colorado 80302 USA.
All rights reserved.

This file and related documentation are protected by copyright and
are distributed under licenses regarding their use, copying, distribution,
and decompilation. No part of this product or related documentation may
be reproduced or transmitted in any form or by any means, electronic or
mechanical, for any purpose, without the express written permission of
Xyleme, Inc.

*/


var MIN_TEXT_WIDTH = 100;
// jQuery collection of HotSpot elements
var $hotSpots = null;

var focusUnbindedElm = null;

var imageMaps = {
  // colection of all ImageMap
  collection : [],
  // add ImageMap object into collection
  addToCollection : function (imageMap){
    this.collection[this.collection.length] = imageMap;
  },
  // get ImageMap object from collection by overlayId
  getById : function(overlayId){
    for(var i = 0; i < this.collection.length; i++) {
      if (this.collection[i].overlayId == overlayId) {
        return  this.collection[i];
      }
    }
    return null;
  },
  // get ImageMap object from collection by index
  getByIndex : function(index){
    for(var i = 0; i < this.collection.length; i++) {
      if (this.collection[i].index == parseInt(index)) {
        return  this.collection[i];
      }
    }
    return null;
  },
  // get ImageMap object from collection by child element
  getByChild : function(child){
    return this.getById(jQuery(child).parents("div[id ^= 'outlinemap' ]").attr("id"));
  }
}

// Constructor
function ImageMap(id, index, overlayId, outlineColor, DISPLAY_AREA_SPECIFIED, SPOT_OUTLINE, hotSpotEvent, mustBeCompleted){
  this.id = id;  
  this.index = parseInt(index);  
  this.overlayId = overlayId;
  this.outlineColor = outlineColor;
  this.DISPLAY_AREA_SPECIFIED = DISPLAY_AREA_SPECIFIED;
  this.SPOT_OUTLINE = SPOT_OUTLINE;
  this.hotSpotEvent = hotSpotEvent;
  this.mustBeCompleted = mustBeCompleted;  
  this.offset = getOffset(document.getElementById(overlayId));
  this.outlineMap = null;
  this.setOutLineMap = function(overlayId, outlineColor){    
    if(outlineColor !== '' && outlineColor != 'transparent'){
      this.outlineMap = new jsGraphics(document.getElementById(overlayId));
      this.outlineMap.setColor(outlineColor);
    }
  }
  this.alreadyInitedSpots = [];
}

function initImageMap(id, index, overlayId, outlineColor, DISPLAY_AREA_SPECIFIED, SPOT_OUTLINE, hotSpotEvent, mustBeCompleted, ds_lc){
  // create and initialize ImageMap object
  var imageMap = new ImageMap(id, index, overlayId, outlineColor, DISPLAY_AREA_SPECIFIED, SPOT_OUTLINE, hotSpotEvent, mustBeCompleted);
  // initialize outLineMap, uses for drawing hotSpots
  imageMap.setOutLineMap(overlayId, outlineColor);
  // add ImageMap object into collection
  imageMaps.addToCollection(imageMap);
  if(SPOT_OUTLINE == 'OnLoad'){
    // show LinkSpots when load
     DrawLinkSpot(id);
  }
  // bind handlers
  bindHandlers(id, SPOT_OUTLINE, hotSpotEvent, ds_lc);
}

function DrawLinkSpot(id){
  jQuery("#" + id).find("area[id ^= 'LinkSpot']").each(function(index, selected){
    imDrawLinkSpot(selected.id);  
  });
}

function bindHandlers(id, SPOT_OUTLINE, hotSpotEvent, ds_lc){
  switch (SPOT_OUTLINE) {
    case 'OnLoad':
      if (hotSpotEvent == "Mouseover") {
        jQuery("#" + id).bind({ mouseover : function(e){ imShowHotSpot(e);  hideOutlineMaps(); }});
      } else {
        jQuery("#" + id).bind({ mousedown : function(e){ imShowHotSpot(e);  hideOutlineMaps(); },
                                mouseup : returnFocus });
      }
      break;
    case 'Mouseover':
      if (hotSpotEvent == "Mouseover") {
        jQuery("#" + id).bind({ mouseover : imOnMouseDown });
      } else {
        jQuery("#" + id).bind({ mousedown : imShowHotSpot,
                                mouseup : returnFocus,
                                mouseover : imShowOutlineMap });
      }
      break;
    case 'Click':
      if(ds_lc == 'readme' || ds_lc == 'showme' || ds_lc == 'quizme'){
        jQuery("#" + id).bind({ mousedown : imOnMouseDown });
      } else  { 
              if (hotSpotEvent == "Mouseover") {
                jQuery("#" + id).bind({ mouseover : imShowHotSpot,
                                        mousedown : imShowOutlineMap });
              } else {
                jQuery("#" + id).bind({ mousedown : imOnMouseDown });
              }
      }        
      break;
    default:
      if (hotSpotEvent == "Mouseover") {
        jQuery("#" + id).bind({ mouseover : imShowHotSpot });
      } else {
        jQuery("#" + id).bind({ mousedown : imShowHotSpot,
                                mouseup : returnFocus });
      }
  }
  jQuery(document).bind({ mousedown: cleanPage });
}

// private function, accessible only from checkNavigationRestriction in CommonUtilities
function checkHotSpotVisit()
{
  for(var i = 0; i < imageMaps.collection.length; i++){
    var imageMap = imageMaps.collection[i];
    if(imageMap.mustBeCompleted){
      var allImageMapSpots = jQuery('#' + imageMap.id).find("area[id^='LinkSpot']")
      if(allImageMapSpots.length - imageMap.alreadyInitedSpots.length){
        return 0;
      }
    }
  }
  return 1;
} 

function cleanPage(e){
  e.stopPropagation();
  if(jQuery(e.target).parents(".ImageMap").length == 0){
    hideHotSpot();
    hideOutlineMaps();
  }
}

function hideHotSpot() {
	//TODO: Add multi flash objects support (on the same page in IE)
	var flashMovie = getFlashMovieObject("FlashMovie");
	
	$hotSpots.each(function(i, selected){
		var elem = document.getElementById('div' + selected.id);
		if (elem != null && typeof(elem) != 'undefined') {
			elem.style.visibility = 'hidden';
			elem.style.display = 'none';

			if (flashMovie != null && typeof(flashMovie) != 'undefined') {
				for (var k=0; k < flashMovie.length; k++) {
					if (isDecendant(flashMovie[k], elem)) {
						try {
							flashMovie[k].Rewind();
						}
						catch(err) {
						}
					}
				}
			}
		}
	});
}

function hideOutlineMaps(){
  for (var i = 0; i < imageMaps.collection.length; i += 1) {
    var imageMap = imageMaps.collection[i];
    if(imageMap.outlineMap != null && imageMap.SPOT_OUTLINE.toLowerCase() != 'onload' ){
      imageMap.outlineMap.clear();
    }
  }
}

function paintOutlineMaps(target){
	var coordsmas = target.coords;
	if (coordsmas == null || typeof(coordsmas) == 'undefined') return;
	var coords = coordsmas.split(',');
	var imageMap = imageMaps.getByChild(target);
	
	if(target.shape.toLowerCase() == "rect") {
		imageMap.outlineMap.drawRect(parseInt(coords[0]) + parseInt(imageMap.offset), parseInt(coords[1]), coords[2]-coords[0], coords[3]-coords[1]);
	}
	if(target.shape.toLowerCase() == "circle")
		imageMap.outlineMap.drawEllipse((coords[0]-coords[2]) + parseInt(imageMap.offset), coords[1]-coords[2], 2 * parseInt(coords[2]), 2 * parseInt(coords[2]));
	if(target.shape.toLowerCase() == "poly") {
		var xs = new Array();
		for(var i=0; i<coords.length/2; i++)
			xs[i] = parseInt(coords[2*i]) + parseInt(imageMap.offset);
			var ys = new Array();
			for(var i=0; i<coords.length/2; i++)
				ys[i] = parseInt(coords[2*i+1]);
				imageMap.outlineMap.drawPolygon(xs, ys);
			}
		imageMap.outlineMap.paint();
}

function showElenemtContent(element,posx,posy){
  var imageMap = imageMaps.getByChild(element);
	if (imageMap.DISPLAY_AREA_SPECIFIED) {
		element.className= 'showedContent_' + imageMap.index + ' ImageMapPopup';
		element.style.display = 'block';
		element.style.visibility = 'visible';
		if (jQuery.inArray(element.id, imageMap.alreadyInitedSpots) == -1) {
			imageMap.alreadyInitedSpots.push(element.id);
		}
	} else {
		element.style.display = 'block';
		element.style.visibility = 'visible';
		element.style.left = posx - jQuery(element).offsetParent().offset().left + "px";
		element.style.top = posy - jQuery(element).offsetParent().offset().top + "px";
		if (jQuery.inArray(element.id, imageMap.alreadyInitedSpots) == -1) {
			imageMap.alreadyInitedSpots.push(element.id);
		}
	}
}

function playFlashMovie(){
	var flashMovie = getFlashMovieObject("FlashMovie");
	if (flashMovie != null && typeof(flashMovie) != 'undefined') {
		if (flashMovie != null && typeof(flashMovie) != 'undefined') {
			for (var k=0; k < flashMovie.length; k++) {
				if (isDecendant(flashMovie[k], element)) {					
					try {
						flashMovie[k].Play();
					}
					catch(err) {
					}
				}
			}
		}
	}
}

function imOnMouseDown(e)
{
	hideHotSpot();
	hideOutlineMaps();
	var target = e.target;
	var id = target.id;
	if (typeof(id) == "undefined" || id.indexOf("LinkSpot") != 0) {
		return;
	}
	var imageMap = imageMaps.getByChild(target); 
	id = 'div' + id;
	var element = document.getElementById(id);
	if(typeof(element) == 'undefined') {
		return;
	}
	var posx = e.pageX + 10;
	var posy = e.pageY + 10;
	var width = target.offsetWidth;
	if (element.innerHTML.length > MIN_TEXT_WIDTH && width < MIN_TEXT_WIDTH) {
		width = MIN_TEXT_WIDTH * 2;
	}
	showElenemtContent(element,posx,posy);

	if(imageMap.outlineMap != null) {
		imageMap.outlineMap.clear();
		paintOutlineMaps(target);
	}	
	// To play flash under IE
	playFlashMovie();
}


function imShowOutlineMap(e)
{
    hideOutlineMaps();
    var target = e.target;
    var imageMap = imageMaps.getByChild(target);
		if (target.parentNode && target.parentNode.name){
			if(imageMap.outlineMap != null) {
				imageMap.outlineMap.clear();
				paintOutlineMaps(target);
			}
			// To play flash under IE
			playFlashMovie();
		}
}

function imShowHotSpot(e)
{
	hideHotSpot();
	var target = e.target;
	var id = target.id;
	if (typeof(id) == "undefined" || id.indexOf("LinkSpot") != 0)
		return;
	id = 'div' + id;
	var element = document.getElementById(id);
	if(typeof(element) == 'undefined') {
		return;
	}
	var posx = e.pageX + 10;
	var posy = e.pageY + 10;
	var width = target.offsetWidth;		
	if (element.innerHTML.length > MIN_TEXT_WIDTH && width < MIN_TEXT_WIDTH)
		width = MIN_TEXT_WIDTH * 2;
	focusUnbindedElm = jQuery(target).unbind("focus").attr('id');  // unbind onfocus event and save id to bind it back
	showElenemtContent(element,posx,posy);
	// To play flash under IE
	playFlashMovie();
}


function imDrawLinkSpot(id)
{
  var target = document.getElementById(id);
	if (typeof(target) == "undefined" || target == null) return;
	var imageMap = imageMaps.getByChild(target);
	if(imageMap.outlineMap != null) {
		paintOutlineMaps(target);
	}
}


function showContent(elemID) {	
	
	// NOT IMPLEMENTED FOR MOUSE OVER

/*	var element = document.getElementById(elemID);
	if(typeof(element) == 'undefined')	
	    return;
	
	hideAll();
	
	if (DISPLAY_AREA_SPECIFIED != null && DISPLAY_AREA_SPECIFIED) {
			element.className= 'showedContent';
			element.style.display = 'block';
			element.style.visibility = 'visible';
		} else {		
			element.style.display = 'block';
			element.style.visibility = 'visible';
			element.style.left = posx + "px";		
			element.style.top = posy + "px";		
			element.style.width = width + "px";
		}*/
}

function getOffset(element, wrap) {
	if(element.children) {
		for(var i=0; i<element.children.length; i++)
		{
			if(element.children[i].tagName == "IMG")
			{
				return element.children[i].offsetLeft;
			}
		}
	} else {
		var children = element.childNodes;
		for(var i=0; i<element.childNodes.length; i++)
		{
			if(element.childNodes[i].tagName == "IMG")
			{
				return element.childNodes[i].offsetLeft;
			}
		}
	}
	return 0;
}

function returnFocus(event){
	if(focusUnbindedElm){
		jQuery('#' + focusUnbindedElm).bind('focus', areaFocus);
		focusUnbindedElm = null;
	}
}

function areaFocus(event){
	hideHotSpot();
	var target = event.currentTarget;
	var id = target.id;
	if (typeof(id) == "undefined" || id.indexOf("LinkSpot") != 0) {
		return;
	}
	var imageMap = imageMaps.getByChild(target);
	id = 'div' + id;
	if (imageMap.DISPLAY_AREA_SPECIFIED) {
		showElenemtContent(document.getElementById(id));
	} else {
		var x = target.coords.split(',');
		var imgId = "#" + "mapImg" + document.getElementById(target.id).parentNode.id.substring(3);
		jQuery("#" + id).css({
			left : findCenterPoly(x, 0) + 'px',
			top :  findCenterPoly(x, 1) + 'px',
			display : 'block',
			visibility : 'visible'
		});
	}

	if(imageMap.SPOT_OUTLINE.toLowerCase() != "onload" && imageMap.SPOT_OUTLINE.toLowerCase() != "never" && imageMap.SPOT_OUTLINE.toLowerCase() != "" 
		&& imageMap.outlineMap != null) {
			imageMap.outlineMap.clear();
			paintOutlineMaps(target);
	}

	// To play flash under IE
	playFlashMovie();
}

jQuery(document).ready(function(){
	jQuery('.ImageMapBody area').bind({ focus : areaFocus });
	// get count of all hot spots on page
	$hotSpots = jQuery("area[id ^= 'LinkSpot']");
});